package com.huimv.eartag2.api.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.eartag2.common.utils.Result;
import com.huimv.eartag2.common.utils.ResultCode;
import com.huimv.eartag2.api.mapper.BizBaseStageMapper;
import com.huimv.eartag2.api.pojo.BizBaseStage;
import com.huimv.eartag2.api.service.BizBaseStageService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zn
 * @since 2022-07-23
 */
@Service
public class BizBaseStageServiceImpl extends ServiceImpl<BizBaseStageMapper, BizBaseStage> implements BizBaseStageService {

    @Resource
    private BizBaseStageMapper bizBaseStageMapper;

    @Override
    public Result addBizBaseStage(Map<String, String> param) {
        BizBaseStage bizBaseStage = new BizBaseStage();
        bizBaseStage.setSort(param.get("sort"));
        bizBaseStage.setStageCode(param.get("stageCode"));
        bizBaseStage.setStageName(param.get("stageName"));
        return new Result(ResultCode.SUCCESS,bizBaseStageMapper.insert(bizBaseStage));
    }

    @Override
    public Result removeBizBaseStage(List<String> ids) {
        return new Result(ResultCode.SUCCESS,bizBaseStageMapper.deleteBatchIds(ids));
    }

    @Override
    public Result updateBizBaseStage(Map<String, String> param) {
        String id = param.get("id");
        BizBaseStage bizBaseStage = bizBaseStageMapper.selectById(id);
        bizBaseStage.setSort(param.get("sort"));
        bizBaseStage.setStageCode(param.get("stageCode"));
        bizBaseStage.setStageName(param.get("stageName"));
        return new Result(ResultCode.SUCCESS,bizBaseStageMapper.updateById(bizBaseStage));
    }

    @Override
    public Result listBizBaseStage(Map<String, String> param) {
        String pageNo = param.get("pageNo");
        String pageSize = param.get("pageSize");
        String stageCode = param.get("stageCode");
        String stageName = param.get("stageName");

        Page<BizBaseStage> page = new Page<>(Integer.valueOf(pageNo), Integer.valueOf(pageSize));
        QueryWrapper<BizBaseStage> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(StringUtils.isNotBlank(stageCode),"stage_code", stageCode);
        queryWrapper.like(StringUtils.isNotBlank(stageName),"stage_name", stageName);
        Page<BizBaseStage> bizBaseStagePage = bizBaseStageMapper.selectPage(page, queryWrapper);
        return new Result(ResultCode.SUCCESS,bizBaseStagePage);
    }

    @Override
    public Result getDataById(String id) {
        return new Result(ResultCode.SUCCESS,bizBaseStageMapper.selectById(id));
    }
}
