package com.huimv.eartag2.api.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.huimv.eartag2.common.utils.DateUtil;
import com.huimv.eartag2.common.utils.Result;
import com.huimv.eartag2.common.utils.ResultCode;
import com.huimv.eartag2.api.pojo.EartagResetCount;
import com.huimv.eartag2.api.mapper.EartagResetCountMapper;
import com.huimv.eartag2.api.service.EartagResetCountService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.util.*;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zn
 * @since 2022-08-12
 */
@Service
public class EartagResetCountServiceImpl extends ServiceImpl<EartagResetCountMapper, EartagResetCount> implements EartagResetCountService {

    @Autowired
    private EartagResetCountMapper eartagResetCountMapper;

    @Override
    public Result getEartagResetCount(String farmCode, String startDate, String endDate, String earmark) throws ParseException {
        QueryWrapper<EartagResetCount> queryWrapper= new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(farmCode),"farm_code",farmCode);
        queryWrapper.between("add_date",startDate,endDate);
        queryWrapper.eq(StringUtils.isNotBlank(earmark),"earmark",earmark);
        queryWrapper.orderByAsc("add_date");
        List<EartagResetCount> eartagResetCountList = eartagResetCountMapper.selectList(queryWrapper);
        List resultList = new ArrayList();
        DateUtil du = new DateUtil();
        for(EartagResetCount artagResetCount:eartagResetCountList){
            Map map = new HashMap<>();
            resultList.add(map);
            map.put("addDate",du.formatDateText(artagResetCount.getAddDate()));
            map.put("times",artagResetCount.getTimes());
        }
        return new Result(ResultCode.SUCCESS,resultList);
    }

    @Override
    public Result listEartagResetCount(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String earmark = paramsMap.get("earmark");
        String day = paramsMap.get("day");
        int days = Integer.parseInt(day);
        List<EartagResetCount> list = new ArrayList<>();
        DateTime dateTime = cn.hutool.core.date.DateUtil.beginOfDay(new Date());
        for (int i = 0; i < days; i++) {
            DateTime beforeDate = cn.hutool.core.date.DateUtil.offsetDay(dateTime, -i);
            EartagResetCount one = this.getOne(new QueryWrapper<EartagResetCount>().eq(StringUtils.isNotBlank(farmId), "farm_code", farmId).eq("add_date", beforeDate).like("earmark", earmark));
            if (ObjectUtil.isEmpty(one)){
                one = new EartagResetCount();
                one.setAddDate(beforeDate);
                one.setFarmCode(farmId);
                one.setEarmark(earmark);
                one.setTimes(0);
            }
            list.add(one);
        }
        return  new Result(ResultCode.SUCCESS,list);
    }
}
