package com.huimv.eartag2.api.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.eartag2.common.utils.DateUtil;
import com.huimv.eartag2.common.utils.Result;
import com.huimv.eartag2.common.utils.ResultCode;
import com.huimv.eartag2.api.pojo.FarmerAdopt;
import com.huimv.eartag2.api.mapper.FarmerAdoptMapper;
import com.huimv.eartag2.api.pojo.FarmerAsset;
import com.huimv.eartag2.api.service.FarmerAdoptService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.eartag2.api.service.FarmerAssetService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.sql.Date;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zn
 * @since 2022-08-29
 */
@Service
public class FarmerAdoptServiceImpl extends ServiceImpl<FarmerAdoptMapper, FarmerAdopt> implements FarmerAdoptService {
    @Autowired
    private FarmerAdoptMapper farmerAdoptMapper;
//    @Autowired
//    private DateUtil dateUtil;
    @Autowired
    private FarmerAssetService farmerAssetService;

    @Override
    public Result add(Map<String, Object> paramsMap) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        //
        String farmCode = paramsMap.get("farmCode")+"";
        String batchCode = paramsMap.get("batchCode")+"";
        Date date = new Date(dateUtil.parseDate(paramsMap.get("adoptDate")+"").getTime());
        Integer adoptAge = Integer.parseInt(paramsMap.get("adoptAge")+"");
        Integer days = Integer.parseInt(paramsMap.get("days")+"");
        Integer quantity = Integer.parseInt(paramsMap.get("quantity")+"");
        BigDecimal weightBd = new BigDecimal(paramsMap.get("weight")+"");
        BigDecimal pigPriceBd = new BigDecimal(paramsMap.get("pigPrice")+"");
        BigDecimal planIncomeBd = new BigDecimal(paramsMap.get("planIncome")+"");
        FarmerAdopt farmerAdopt = new FarmerAdopt();
        farmerAdopt.setBatchCode(batchCode);
        farmerAdopt.setAdoptDate(date);
        farmerAdopt.setAdoptAge(adoptAge);
        farmerAdopt.setDays(days);
        farmerAdopt.setQuantity(quantity);
        farmerAdopt.setWeight(weightBd);
        farmerAdopt.setPigPrice(pigPriceBd);
        farmerAdopt.setPlanIncome(planIncomeBd);
        farmerAdopt.setFarmCode(farmCode);
        int rows = farmerAdoptMapper.insert(farmerAdopt);
        if(rows > 0){
            return new Result(ResultCode.SUCCESS);
        }else{
            return new Result(ResultCode.FAIL);
        }
    }

    @Override
    public Result edit(Map<String, Object> paramsMap) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        //
        String batchCode = paramsMap.get("batchCode")+"";
        Date date = new Date(dateUtil.parseDate(paramsMap.get("adoptDate")+"").getTime());
        Integer id = Integer.parseInt(paramsMap.get("id")+"");
        Integer adoptAge = Integer.parseInt(paramsMap.get("adoptAge")+"");
        Integer days = Integer.parseInt(paramsMap.get("days")+"");
        Integer quantity = Integer.parseInt(paramsMap.get("quantity")+"");
        BigDecimal weightBd = new BigDecimal(paramsMap.get("weight")+"");
        BigDecimal pigPriceBd = new BigDecimal(paramsMap.get("pigPrice")+"");
        BigDecimal planIncomeBd = new BigDecimal(paramsMap.get("planIncome")+"");

        FarmerAdopt farmerAdopt = new FarmerAdopt();
        farmerAdopt.setId(id);
        farmerAdopt.setBatchCode(batchCode);
        farmerAdopt.setAdoptDate(date);
        farmerAdopt.setAdoptAge(adoptAge);
        farmerAdopt.setDays(days);
        farmerAdopt.setQuantity(quantity);
        farmerAdopt.setWeight(weightBd);
        farmerAdopt.setPigPrice(pigPriceBd);
        farmerAdopt.setPlanIncome(planIncomeBd);
        int rows = farmerAdoptMapper.updateById(farmerAdopt);
        if(rows > 0){
            return new Result(ResultCode.SUCCESS);
        }else{
            return new Result(ResultCode.FAIL);
        }
    }

    @Override
    public Result remove(Map<String, Object> paramsMap) {
        String ids = paramsMap.get("ids")+"";
        List<Integer> idList = new ArrayList<>();
        String[] idArray = ids.split(",");
        for(int a=0;a<idArray.length;a++){
            idList.add(Integer.parseInt(idArray[a]));
        }
        int rows = farmerAdoptMapper.deleteBatchIds(idList);
        if(rows > 0){
            return new Result(ResultCode.SUCCESS);
        }else{
            return new Result(ResultCode.FAIL);
        }
    }

    @Override
    public Result listByFarmCode(Map<String, Object> paramsMap) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        //
        String farmCode = paramsMap.get("farmCode")+"";
        String batchCode = paramsMap.get("batchCode")+"";
        Date startDate = new Date(dateUtil.parseDate(paramsMap.get("startDate")+"").getTime());
        Date endDate = new Date(dateUtil.parseDate(paramsMap.get("endDate")+"").getTime());
        Integer pageNo = Integer.parseInt(paramsMap.get("pageNo")+"");
        Integer pageSize = Integer.parseInt(paramsMap.get("pageSize")+"");
        QueryWrapper<FarmerAdopt> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code",farmCode);
        queryWrapper.eq(StringUtils.isNotBlank(batchCode),"batch_code",batchCode);
        queryWrapper.between("adopt_date",startDate,endDate);
        Page<FarmerAdopt> page = new Page<>(Integer.valueOf(pageNo), Integer.valueOf(pageSize));
        return new Result(ResultCode.SUCCESS,farmerAdoptMapper.selectPage(page,queryWrapper));
    }

    @Override
    public Result getLastAdoptByFarmCode(Map<String, Object> paramsMap) {
        //
        String farmCode = paramsMap.get("farmCode")+"";
        String manyMonth = paramsMap.get("manyMonth")+"";
        //{获取最新一个批次的数据}
        FarmerAdopt farmerAdopt = farmerAdoptMapper.getLastAdoptByFarmCode(farmCode);
        //{获取资产历史数据}
        //-------------------------------------------------------
        List<FarmerAsset> farmerAssetList = farmerAssetService.getLastManyMonthAssets(farmCode,manyMonth);
        JSONObject resultJo = new JSONObject();
        resultJo.put("adopt",farmerAdopt);
        resultJo.put("asset",farmerAssetList);
        return new Result(ResultCode.SUCCESS,resultJo);
    }

}
