package com.huimv.eartag2.device.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.huimv.eartag2.common.dao.entity.*;
import com.huimv.eartag2.common.dao.repo.*;
//import com.huimv.eartag2.process.service.IDeviceService;
//import com.huimv.eartag2.process.utils.DateUtil;
//import com.huimv.eartag2.process.utils.MathUtil;
import com.huimv.eartag2.common.service.IBaseConfigService;
import com.huimv.eartag2.common.utils.BizConst;
import com.huimv.eartag2.common.vo.FarmAllStatusVo;
import com.huimv.eartag2.device.service.ICacheService;
import com.huimv.eartag2.device.service.IDeviceService;
import com.huimv.eartag2.device.service.IEartagService;
import com.huimv.eartag2.device.utils.DateUtil;
import com.huimv.eartag2.device.utils.MathUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Example;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.math.BigDecimal;
import java.sql.Timestamp;
import java.text.ParseException;
import java.util.*;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
@Slf4j
public class DeviceServiceImpl implements IDeviceService {
    @Autowired
    private MathUtil mathUtil;
    @Autowired
    private DateUtil dateUtil;
    @Autowired
    private EartagRawRepo rawRepo;
    @Autowired
    private EartagHeartbeatRepo heartbeatRepo;
    @Autowired
    private EartagEnvRepo envRepo;
//    @Autowired
//    private IDeviceService deviceService;
    @Autowired
    private ICacheService cacheService;
    @Autowired
    private EartagDeviceRegisterRepo deviceRegisterRepo;
    @Autowired
    private EartagDeviceOnlineRepo deviceOnlineRepo;
    @Autowired
    private EartarFarmAllStatusRepo eartagFarmAllStatusRepo;
    @Autowired
    private EartarFarmAllStatusRepo farmAllStatusRepo;
    @Autowired
    private IEartagService eartagService;
    @Autowired
    private EartagDeviceOnlineRepo eartagDeviceOnlineRepo;
    @Autowired
    private EartagEartagRegisterRepo eartagRegisterRepo;
    @Autowired
    private EartagConfigPropertiesRepo configPropertiesRepo;
    @Value("${service.farmAllStatus.host}")
    private String serviceFarmAllStatusHost;
    @Value("${service.farmAllStatus.ip}")
    private String serviceFarmAllStatusIp;
    @Value("${service.farmAllStatus.url}")
    private String serviceFarmAllStatusUrl;
    @Autowired
    private IBaseConfigService iBaseConfigService;
    @Autowired
    private BasePigpenEntityRepo basePigpenRepo;
    @Autowired
    private EartagAbnormalAlarmEntityRepo eartagAbnormalAlarmEntityRepo;
    @Autowired
    private EartagAbnormalCountEntityRepo eartagAbnormalCountEntityRepo;
    @Autowired
    private EartagDeviceRegisterRepo eartagDeviceRegisterRepo;

    /**
     * @Method      : handleDeviceHeartbeat
     * @Description : 处理设备心跳数据
     * @Params      : [dataJo]
     * @Return      : void
     *
     * @Author      : ZhuoNing
     * @Date        : 2022/3/12
     * @Time        : 19:16
     */
    @Override
    public void handleDeviceHeartbeat(JSONObject dataJo) throws ParseException {
        System.out.println("心跳数据 dataJo>>"+dataJo);
        //逻辑处理：
        //1、保存设备心跳流水数据；
        //2、更新设备注册表（活动状态、活动更新时间，更新类型）
        //3、更新在线统计表（心跳数，心跳最早上传时间，最后一次上传时间）
        //4、更新总状态表表
        //Step1:更新设备注册表[设置活动状态为1，设置活动状态更新时间]
        //Step2:更新在线统计[心跳数+1，心跳第一次上传时间，心跳最后一次时间]，预先判断该数据是否存在，如果不存在要先添加记录，如果存在就修改数据

        System.out.println("dataJo>"+dataJo);
        String deviceCode = dataJo.getString("device");
        //{获取牧场id}
        String farmId = getFarmIdByDeviceCode(deviceCode);
        if(farmId != null){
            String todayDateText = new com.huimv.eartag2.common.utils.DateUtil().getTodayDateText();
            java.sql.Date todayDate = new java.sql.Date(new java.util.Date().getTime());
            Timestamp nowTimestamp = new Timestamp(new java.util.Date().getTime());
            //1、{保存设备心跳记录}
            saveDeviceHeartbeat(dataJo,todayDate);

            String updateType="设备心跳";
            //2、{更新设备注册表}
            updateDeviceRegister(farmId, deviceCode,nowTimestamp,updateType);

            //3、{更新在线统计表}
            updateDeviceOnlineCountWithHeartbeat(farmId, deviceCode,todayDateText,nowTimestamp,todayDate);

            //判断该设备编号是否存在缓存在线集合当中；
            Boolean isExist = cacheService.isExistDeviceOnlineSet(farmId, deviceCode, todayDateText);
            System.out.println("心跳处理 isExist>>"+isExist);
            if (!isExist) {
                //4、{更新总状态表}
                updateEnvtempOfDeviceAllStatus(farmId, todayDateText,todayDate,nowTimestamp);

                //# 将新设备编号加入在线缓存 #
                cacheService.putDeviceToOnlineSet(farmId, deviceCode,todayDateText);
            }
        }else{
            log.error("出错:该请求的FarmId为空.请检查该设备编码["+deviceCode+"]是否存在FarmId.");
        }
        System.out.println("## END 心跳数据。 "+new com.huimv.eartag2.common.utils.DateUtil().getTodayMissionText());
    }

    //更新总状态
    public void updateDeviceAllStatus(String farmId, String deviceCode, String todayDateText, Timestamp nowTimestamp, java.sql.Date todayDate) throws ParseException {
        System.out.println("## 心跳 2.3");
        //读取一条总状态记录；
        EartarFarmAllStatusEntity farmAllStatusEntity = farmAllStatusRepo.getOneByFarmIdAndCreateDate(farmId,todayDateText);
        if(farmAllStatusEntity == null){
            System.out.println("## 心跳 2.3.1");
            //总状态记录不存在；
            //#创建一条新总状态记录；
            //*计算设备在线数量、离线数量、在线率*
            int deviceTotal = getDeviceTotalByFarmIdFromDeviceRegister(farmId);
            int onlineNum = 1;
            int offlineNum = deviceTotal-1;
            BigDecimal reateBd= new BigDecimal(onlineNum).divide(new BigDecimal(deviceTotal),1,BigDecimal.ROUND_HALF_UP);
            //*计算设备注销数量*
            int cancelNum = getDeviceCancelTotalFromDeviceRegister(farmId);
            //#计算耳标总数
            int eartagTotal = eartagService.getEartagRegTotalByFarmId(farmId);
            //计算耳标在线数量
            int eartagOnlineTotal = eartagService.getEartagOnelineTotal(farmId,todayDateText);
            //
            EartarFarmAllStatusEntity newFarmAllStatusEntity = new EartarFarmAllStatusEntity();
            newFarmAllStatusEntity.setDeviceTotal(deviceTotal);
            newFarmAllStatusEntity.setDeviceOnline(onlineNum);
            newFarmAllStatusEntity.setDeviceOffline(offlineNum);
            newFarmAllStatusEntity.setDeviceRate(Float.parseFloat(reateBd.toString()));
            newFarmAllStatusEntity.setDeviceCancel(cancelNum);
            newFarmAllStatusEntity.setEartagTotal(eartagTotal);
            newFarmAllStatusEntity.setEartagOnline(eartagOnlineTotal);
            newFarmAllStatusEntity.setFarmId(farmId);
            newFarmAllStatusEntity.setUpdateTime(nowTimestamp);
            newFarmAllStatusEntity.setCreateDate(todayDate);
            newFarmAllStatusEntity.setFarmId(farmId);
            farmAllStatusRepo.saveAndFlush(newFarmAllStatusEntity);
            //更新总状态表缓存
            cacheService.putAllFarmAllStatusCache(farmId,newFarmAllStatusEntity);
        }else{
            System.out.println("## 心跳 2.3.2");
            //总状态记录存在；
            //#计算设备的在线、离线、在线率，更新时间
            int onlineNum = farmAllStatusEntity.getDeviceOnline();
            onlineNum++;
            //
            int total = getDeviceTotalByFarmIdFromDeviceRegister(farmId);
            int offlineNum = 0;
            if(total > onlineNum){
                offlineNum = total - onlineNum;
            }
            farmAllStatusEntity.setDeviceTotal(total);
            BigDecimal reateBd= new BigDecimal(onlineNum).divide(new BigDecimal(total),1,BigDecimal.ROUND_HALF_UP);
            farmAllStatusEntity.setDeviceOnline(onlineNum);
            farmAllStatusEntity.setDeviceOffline(offlineNum);
            farmAllStatusEntity.setDeviceRate(Float.parseFloat(reateBd.toString()));
            farmAllStatusEntity.setUpdateTime(nowTimestamp);
            eartagFarmAllStatusRepo.saveAndFlush(farmAllStatusEntity);
            //更新总状态表缓存
            cacheService.putAllFarmAllStatusCache(farmId,farmAllStatusEntity);
        }
    }

    //计算设备总数
    public int getDeviceTotalByFarmIdFromDeviceRegister(String farmId){
        //从缓存钟读取设备总数
        Integer deviceTotal = cacheService.getDeviceTotal(farmId);
        if(deviceTotal != null && deviceTotal != 0) {
            return deviceTotal;
        }else{
//            List<Object[]> deviceRegList = deviceRegisterRepo.getDeviceCountByFarmId(farmId);
//            System.out.println("deviceRegList.size>>>>"+deviceRegList.size());
//            Object[] deviceRegObj = (Object[]) deviceRegList.get(0);
//            Integer total = Integer.parseInt(deviceRegObj[0].toString());
            Integer total = getEffectiveDeviceSum(farmId);
            //设置缓存
            cacheService.putDeviceTotal(total,farmId);
            //设备总数
            return total;
        }
    }

    //计算注销设备数量
    private int getDeviceCancelTotalFromDeviceRegister(String farmId) {
        //从缓存钟读取设备总数
        Integer deviceCancel = cacheService.getDeviceCancel(farmId);
        if(deviceCancel != null) {
            return deviceCancel;
        }else {
            //注销状态
            Integer liveStatus = 2;
            List<Object[]> deviceCancelList = deviceRegisterRepo.getDeviceCountByFarmIdAndLiveStatus(farmId,liveStatus);
            Object[] deviceRegObj = (Object[]) deviceCancelList.get(0);
            Integer total = Integer.parseInt(deviceRegObj[0].toString());
            //设置注销缓存
            cacheService.putDeviceCancel(total,farmId);
            //设备总数
            return total;
        }
    }

    //更新在线状态
    public void updateDeviceOnlineCountWithHeartbeat(String farmId, String deviceCode, String todayDateText, Timestamp nowTimestamp, java.sql.Date todayDate) throws ParseException {
        String hashKey = BizConst.DEVICE_REGISTER_PREFIX+deviceCode;
        //计算设备别名
        String alias = (String)cacheService.getHashProperty(hashKey,"deviceAlias");
        //读取在线日期
        EartagDeviceOnlineEntity eartagDeviceOnlineEntity = eartagDeviceOnlineRepo.getDeviceOnline(deviceCode, todayDateText);
        if (eartagDeviceOnlineEntity == null) {
            Integer heartbeatTotal = 1;
            Integer envtempTotal = 0;
            Timestamp heartbeatFirstTime = nowTimestamp;
            Timestamp heartbeatLastTime = nowTimestamp;
            Timestamp envtempFirstTime = null;
            Timestamp envtempLastTime = null;
            Timestamp firstTime = nowTimestamp;
            Timestamp lastTime = nowTimestamp;
            Integer liveStatus = 1;
            //{计算耳标总数}
//            Integer eartagTotal = eartagService.getEartagRegTotalByFarmId(farmId);
            Integer eartagTotal = 0;
            //{新建设备在线统计并创建缓存}
            newDeviceOnline(deviceCode,alias,heartbeatTotal,envtempTotal,eartagTotal,todayDate,farmId,heartbeatFirstTime,heartbeatLastTime,envtempFirstTime,envtempLastTime,firstTime,lastTime,liveStatus);
        }else{
            eartagDeviceOnlineEntity.setAlias(alias);
            int heartbeatTotal = eartagDeviceOnlineEntity.getHeartbeatTotal();
            eartagDeviceOnlineEntity.setHeartbeatTotal(++heartbeatTotal);
            if(eartagDeviceOnlineEntity.getHeartbeatFirstTime() == null){
                eartagDeviceOnlineEntity.setHeartbeatFirstTime(nowTimestamp);
            }
            eartagDeviceOnlineEntity.setHeartbeatLastTime(nowTimestamp);
            if(eartagDeviceOnlineEntity.getFirstTime() == null){
                eartagDeviceOnlineEntity.setFirstTime(nowTimestamp);
            }
            eartagDeviceOnlineEntity.setLastTime(nowTimestamp);
            eartagDeviceOnlineRepo.saveAndFlush(eartagDeviceOnlineEntity);
            //更新设备在线缓存数据
            cacheService.putDeviceInOnline(deviceCode,eartagDeviceOnlineEntity);
        }
    }

    //更新在线状态
    public void updateDeviceOnlineCount(String farmId, String deviceCode, String todayDateText, Timestamp nowTimestamp, java.sql.Date todayDate) throws ParseException {
        //
        EartagDeviceOnlineEntity deviceOnlineEntity = deviceOnlineRepo.getDeviceOnline(deviceCode,todayDateText);
        if(deviceOnlineEntity == null){
            //不存在；
            EartagDeviceOnlineEntity newDeviceOnlineEntity = new EartagDeviceOnlineEntity();
            newDeviceOnlineEntity.setDeviceCode(deviceCode);
            newDeviceOnlineEntity.setHeartbeatTotal(1);
            newDeviceOnlineEntity.setEnvtempTotal(0);
            newDeviceOnlineEntity.setEartagTotal(0);
            newDeviceOnlineEntity.setAddDate(todayDate);
            newDeviceOnlineEntity.setHeartbeatFirstTime(nowTimestamp);
            newDeviceOnlineEntity.setHeartbeatLastTime(nowTimestamp);
            newDeviceOnlineEntity.setFarmId(farmId);
            deviceOnlineRepo.saveAndFlush(newDeviceOnlineEntity);
            //更新每天在线统计缓存
            cacheService.putDeviceInOnline(deviceCode,newDeviceOnlineEntity);
        }else{
            System.out.println("## 心跳 1.2.2/2.2.2");
            //存在；
            int total = deviceOnlineEntity.getHeartbeatTotal();
            deviceOnlineEntity.setHeartbeatTotal(++total);
            deviceOnlineEntity.setHeartbeatLastTime(nowTimestamp);
            deviceOnlineRepo.saveAndFlush(deviceOnlineEntity);
            //更新每天在线统计缓存
            cacheService.putDeviceInOnline(deviceCode,deviceOnlineEntity);
        }

        //--（优化思路） --//
//        updateDeviceOnlineCount(deviceCode);

    }

    //更新设备注册表
    private void updateDeviceRegister(String farmId, String deviceCode, Timestamp nowTimestamp,String updateType) {
        //--更新mysql
        EartagDeviceRegisterEntity deviceRegisterEntity = deviceRegisterRepo.getDeviceRegister(deviceCode);
        if (deviceRegisterEntity != null) {
            //设备注册数据存在
            deviceRegisterEntity.setDeviceStatus(1);
            deviceRegisterEntity.setActiveStatus(1);
            deviceRegisterEntity.setActiveTime(nowTimestamp);
            deviceRegisterEntity.setUpdateType(updateType);
            deviceRegisterRepo.saveAndFlush(deviceRegisterEntity);
            //更新缓存中设备注册信息
            cacheService.putDeviceRegister(deviceCode,deviceRegisterEntity);
        }else{
            //设备注册数据不存在
            //新建设备注册记录
            newDeviceRegister(farmId,deviceCode,nowTimestamp,updateType);
        }
    }

    //新建设备注册记录
    public void newDeviceRegister(String farmId,String deviceCode,Timestamp nowTimestamp,String updateType){
        EartagDeviceRegisterEntity newDeviceRegisterEntity = new EartagDeviceRegisterEntity();
        newDeviceRegisterEntity.setDeviceCode(deviceCode);
        newDeviceRegisterEntity.setFarmId(farmId);
        newDeviceRegisterEntity.setRegisterTime(nowTimestamp);
        newDeviceRegisterEntity.setActiveStatus(1);
        newDeviceRegisterEntity.setActiveTime(nowTimestamp);
        newDeviceRegisterEntity.setDeviceStatus(1);
        newDeviceRegisterEntity.setUpdateType(updateType);
        deviceRegisterRepo.saveAndFlush(newDeviceRegisterEntity);
        //更新缓存中设备注册信息
        cacheService.putDeviceRegister(deviceCode,newDeviceRegisterEntity);
    }

    /**
     * @Method      : handleDeviceEnvtemp
     * @Description : 处理设备环境温度数据
     * @Params      : [dataJo]
     * @Return      : void
     *
     * @Author      : ZhuoNing
     * @Date        : 2022/3/12
     * @Time        : 19:16
     */
    @Override
    public void handleDeviceEnvtemp(JSONObject dataJo) throws ParseException {
        System.out.println("环境数据 dataJo>>"+dataJo);
        //逻辑处理：
        //1、保存环境温度流水数据；
        //2、更新设备注册表（活动状态、活动更新时间，更新类型）
        //3、更新在线统计表（环境温度计数，环境温度最早上传时间，最新环境温度，环境温度最后一次上传时间）
        //4、更新总状态表
        String deviceCode = dataJo.getString("device");
        //{通过设备编码取货牧场id}
        String farmId = getFarmIdByDeviceCode(deviceCode);
        if(farmId != null){
            String temp = dataJo.getString("temp");
            String envtemp = com.huimv.eartag2.common.utils.MathUtil.countEnvtemp(temp,2).toString();
            String todayDateText = new com.huimv.eartag2.common.utils.DateUtil().getTodayDateText();
            java.sql.Date todayDate = new java.sql.Date(new java.util.Date().getTime());
            Timestamp nowTimestamp = new Timestamp(new java.util.Date().getTime());

            //1.保存环境温度流水
            saveDeviceTemp(dataJo,todayDate,nowTimestamp);

            String updateType = "环境温度";
            //2.{更新设备注册表}
            updateDeviceRegister(farmId, deviceCode,nowTimestamp,updateType);

            //3.{更新在线统计表}
            updateDeviceOnlineWithEnvtemp(farmId, deviceCode,envtemp,todayDate,nowTimestamp);

            //{检查异常温度}
//            checkAbnormalEnvTemp(farmId,envtemp);
            //--处理异常环境温度报警
            handleEnvtempAlarmAndCount(dataJo,todayDate,nowTimestamp,farmId);

            //判断该设备编号是否存在牧场缓存在线集合当中；
            Boolean isExist = cacheService.isExistDeviceOnlineSet(farmId, deviceCode,todayDateText);
            System.out.println("## 环境温度 isExist>>"+isExist);
            if (!isExist) {
                //4.{更新总状态表}
                updateEnvtempOfDeviceAllStatus(farmId, todayDateText,todayDate,nowTimestamp);

                //{将新设备编号加入在线缓存}
                cacheService.putDeviceToOnlineSet(farmId, deviceCode, todayDateText);
            }
        }else{
            log.error("出错:该请求的FarmId为空.请检查该设备编码["+deviceCode+"]是否存在FarmId.");
        }
        System.out.println("## END 环境数据。"+new com.huimv.eartag2.common.utils.DateUtil().getTodayMissionText());
    }

    //{处理环境温度报警}
    private void handleEnvtempAlarmAndCount(JSONObject dataJo, java.sql.Date todayDate, Timestamp nowTimestamp, String farmId) {
        String device = dataJo.getString("device");
        System.out.println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"+dataJo);
        Map maxAndMinMap = getMaxAndMin2(farmId, "envMaxTem", "envMinTem");
        BigDecimal configMaxTempBd = new BigDecimal(maxAndMinMap.get("envMaxTem").toString());
        BigDecimal configMinTempBd = new BigDecimal(maxAndMinMap.get("envMinTem").toString());
        BigDecimal envTemp1Bd = new BigDecimal(dataJo.getString("temp1"));
        if (envTemp1Bd.compareTo(configMaxTempBd) >= 0) {
            //{}
            String warningObject = getWarningObject(device);
            if(warningObject == null){
                return;
            }
            // 保存耳标高温报警
            EartagAbnormalAlarmEntity eartagAbnormalAlarmEntity = new EartagAbnormalAlarmEntity();
            eartagAbnormalAlarmEntity.setFarmCode(farmId);
            eartagAbnormalAlarmEntity.setContent("高温警报：["+warningObject+"]当前环境温度为" + envTemp1Bd.toString() + "°，高于等于高温阈值" + configMaxTempBd.toString() + "°");
            //数据:2
            eartagAbnormalAlarmEntity.setAlarmType(2);
            //采集器:1
            eartagAbnormalAlarmEntity.setAlarmFrom(1);
            eartagAbnormalAlarmEntity.setAddTime(nowTimestamp);
            eartagAbnormalAlarmEntity.setAddDate(todayDate);
            //环境高温:3
            eartagAbnormalAlarmEntity.setAlarmSort(3);
            eartagAbnormalAlarmEntity.setTempValue(envTemp1Bd.toString());
            eartagAbnormalAlarmEntityRepo.saveAndFlush(eartagAbnormalAlarmEntity);
            // 高温次数记录
            saveEartagTempDataCount(todayDate, nowTimestamp, farmId);
        } else if (configMinTempBd.compareTo(envTemp1Bd) >= 0) {
            //{}
            String warningObject = getWarningObject(device);
            if(warningObject == null){
                return;
            }
            // 保存耳标高温报警
            EartagAbnormalAlarmEntity eartagAbnormalAlarmEntity = new EartagAbnormalAlarmEntity();
            eartagAbnormalAlarmEntity.setFarmCode(farmId);
            eartagAbnormalAlarmEntity.setContent("低温警报：["+warningObject+"]当前环境温度为" + envTemp1Bd.toString() + "°，低于等于低温阈值" + configMinTempBd.toString() + "°");
            //数据:2
            eartagAbnormalAlarmEntity.setAlarmType(2);
            //采集器:1
            eartagAbnormalAlarmEntity.setAlarmFrom(1);
            eartagAbnormalAlarmEntity.setAddTime(nowTimestamp);
            eartagAbnormalAlarmEntity.setAddDate(todayDate);
            //环境低温:4
            eartagAbnormalAlarmEntity.setAlarmSort(4);
            eartagAbnormalAlarmEntity.setTempValue(envTemp1Bd.toString());
            eartagAbnormalAlarmEntityRepo.saveAndFlush(eartagAbnormalAlarmEntity);
            //低温次数记录
            saveEartagTempDataCount(todayDate, nowTimestamp, farmId);
        }
    }

    //--读取报警对象
    public String getWarningObject(String device){
        EartagDeviceRegisterEntity queryEartagDeviceRegisterEntity = eartagDeviceRegisterRepo.getByDeviceCode(device);
        if(queryEartagDeviceRegisterEntity == null){
            log.error("该采集器设备["+device+"]不存在.");
            return null;
        }
        String warningObject = "";
        if(queryEartagDeviceRegisterEntity.getPigpenId()!=null){
            //{读取栋舍}
            warningObject = getPigpenByDevice(device);
        }else{
            warningObject = queryEartagDeviceRegisterEntity.getFarmerName();
        }
        return warningObject;
    }

    // 保存异常耳标温度计数
    public void saveEartagTempDataCount(java.sql.Date todayDate, Timestamp nowTimestamp, String farmCode) {
        EartagAbnormalCountEntity eartagAbnormalCountEntity = new EartagAbnormalCountEntity();
        eartagAbnormalCountEntity.setFarmCode(farmCode);
        eartagAbnormalCountEntity.setAddDate(todayDate);
        Example<EartagAbnormalCountEntity> example = Example.of(eartagAbnormalCountEntity);
        Optional<EartagAbnormalCountEntity> optional = eartagAbnormalCountEntityRepo.findOne(example);
        if (!optional.isPresent()) {
            eartagAbnormalCountEntity.setEartagDataCount(1);
            eartagAbnormalCountEntity.setEartagDataUpateTime(nowTimestamp);
            eartagAbnormalCountEntityRepo.saveAndFlush(eartagAbnormalCountEntity);
        } else {
            EartagAbnormalCountEntity existEartagAbnormalCountEntity = optional.get();
            Integer count = existEartagAbnormalCountEntity.getEartagDataCount();
            if(count == null){
                count = 0;
            }
            existEartagAbnormalCountEntity.setEartagDataCount(count + 1);
            existEartagAbnormalCountEntity.setEartagDataUpateTime(nowTimestamp);
            eartagAbnormalCountEntityRepo.saveAndFlush(existEartagAbnormalCountEntity);
        }
    }
    //--通过设备获取栋舍
    private String getPigpenByDevice(String device) {
        //根据设备读取栋舍
        List<Object[]> pigpenList = basePigpenRepo.getPigpenByDevice(device);
        if(pigpenList.size()==0){
            return "";
        }else{
            StringBuilder sb = new StringBuilder();
            for(int a=0;a<pigpenList.size();a++){
                Object[] objArray = pigpenList.get(0);
                if(sb.length()>0){
                    sb.append(",");
                }
                sb.append(objArray[0]).append(",").append(objArray[1]);
            }
            return sb.toString();
        }
    }

    // 获取最大最小数值
    private Map getMaxAndMin2(String farmCode, String maxKeyName, String minKeyName) {
        Map configMap = new HashMap();
        String hashKey = BizConst.HASH_EARTEMP_TH_PREFIX + farmCode;
        // 从属性配置表取值
        String confMaxTem = getConfigValue(farmCode, maxKeyName);
        if (confMaxTem == null) {
            log.error("属性配置:"+maxKeyName+"未配置,请联系管理员进行配置.");
        }
        configMap.put(maxKeyName, confMaxTem);
        //获取耳标温度阈值：最小值
        // 从属性配置表取值
        String configMinTem = getConfigValue(farmCode, minKeyName);
        if (configMinTem == null) {
            log.error("属性配置:"+minKeyName+"未配置,请联系管理员进行配置.");
        }
        configMap.put(minKeyName, configMinTem);
        return configMap;
    }

    // 从属性配置表取值
    public String getConfigValue(String farmCode, String configKey) {
        //从数据库读取温度阈值属性
//        List<SysBaseConfigEntity> configList = sysBaseConfigEntityRepo.getConfigValue(configKey, farmCode);
//        if (configList.size() > 0) {
//            SysBaseConfigEntity sysBaseConfigEntity = configList.get(0);
//            return sysBaseConfigEntity.getConfigValue();
//        } else {
//            return null;
//        }
        Map configMap = iBaseConfigService.getConfigValue(configKey,farmCode,"0");
        if(configMap == null){
            log.error("采用默认配置参数 configKey="+configKey);
            return null;
        }else{
            return configMap.get("configValue")+"";
        }
    }

    /**
     * @Method      : checkAbnormalEnvTemp
     * @Description : 检查异常环境温度
     * @Params      : [farmId, envtemp]
     * @Return      : void
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/4/6       
     * @Time        : 19:14
     */
    private void checkAbnormalEnvTemp(String farmId, String envtemp) {
        //{}
        Object envTempTvObj = cacheService.getEnvTempThresholdVal(farmId);
        String envTempTv = "";
        if(envTempTvObj != null){
            envTempTv = envTempTvObj.toString();
            String[] envTempArray = envTempTv.split("-");
            if(envTempArray.length==1){
                String lowTemp = envTempArray[0];

            }else  if(envTempArray.length==2){
                String lowTemp = envTempArray[0];
                if(lowTemp == null || lowTemp.trim().length() ==0){
                    
                }
                String highTemp = envTempArray[1];

            }


            //===============================================================================
        }else{
            String sort = "";
            //从数据库读取阈值
            EartagConfigPropertiesEntity configPropertiesEntity = configPropertiesRepo.getThresholdVal(farmId,sort);
            if(configPropertiesEntity != null){
                String lowTemp = configPropertiesEntity.getProVal1();
                String highTemp = configPropertiesEntity.getProVal2();


                //===============================================================================
            }else{
                log.error(farmId+"牧场的环境温度高低温阈值为空.请检查，否则不能进行设备的高低温异常统计.");
            }
        }
    }

    //更新设备环境信息
    @Override
    public void updateEnvtempOfDeviceAllStatus(String farmId, String todayDateText, java.sql.Date todayDate, Timestamp nowTimestamp) throws ParseException {
        System.out.println("## farmId>>"+farmId);
        //{获取处于工作状态的设备数量}
        Integer deviceTotal = getDeviceTotalByFarmIdFromDeviceRegister(farmId);
        System.out.println("## 1.deviceTotal>>"+deviceTotal);
        if(deviceTotal == 0){
            log.error("出错:设备总数(deviceTotal)不能为0.");
        }
        //设备在线数量
        Integer deviceOnlineSum = cacheService.getDeviceOnlineSum(farmId,todayDateText);
        System.out.println("## 2.设备当前在线数量(deviceOnlineSum)="+deviceOnlineSum);
        deviceOnlineSum++;
        System.out.println("## 3.设备当前在线数量(deviceOnlineSum) 2="+deviceOnlineSum);
        //设备离线数量
        Integer deviceOfflineSum = 0;
        if(deviceTotal > deviceOnlineSum){
            deviceOfflineSum = deviceTotal - deviceOnlineSum;
        }
        System.out.println("## 4.deviceOfflineSum>>"+deviceOfflineSum);
        //设备在线率
        BigDecimal deviceRateBd= new BigDecimal(deviceOnlineSum).divide(new BigDecimal(deviceTotal),1,BigDecimal.ROUND_HALF_UP);
        Float deviceOnlineRate = Float.parseFloat(deviceRateBd.toString());
        System.out.println("## 5.deviceOnlineRate>>"+deviceOnlineRate);
        //{获取注销设备数量}
        Integer deviceCancelSum = getDeviceCancelTotalFromDeviceRegister(farmId);
        Integer eartagOnlineSum = 0;
        Float eartagOnlineRate = 0F;
        //耳标总数量
        Integer eartagTotal = eartagService.getEartagRegTotalByFarmId(farmId);
        if(eartagTotal != 0){
            //耳标在线数量
            eartagOnlineSum = eartagService.getEartagOnelineTotal(farmId,todayDateText);
            //耳标在线率
            BigDecimal eartagRateBd= new BigDecimal(eartagOnlineSum).divide(new BigDecimal(eartagTotal),1,BigDecimal.ROUND_HALF_UP);
            eartagOnlineRate = Float.parseFloat(eartagRateBd.toString());
        }
        //
        EartarFarmAllStatusEntity farmAllStatusEntity = eartagFarmAllStatusRepo.getOneByFarmIdAndCreateDate(farmId,todayDateText);
        if(farmAllStatusEntity == null){
            //{新建设备状态记录并更新缓存cache}
            createNewFarmAllStatus(farmId,todayDate,nowTimestamp,deviceTotal,deviceOnlineSum,deviceOfflineSum,deviceCancelSum,deviceOnlineRate,eartagTotal,eartagOnlineSum,eartagOnlineRate);
        }else{
            //计算设备状态并更新缓存cache(在线数+1，离线数-1，计算设备在线率)
            farmAllStatusEntity.setDeviceTotal(deviceTotal);
            farmAllStatusEntity.setDeviceOnline(deviceOnlineSum);
            farmAllStatusEntity.setDeviceOffline(deviceOfflineSum);
            farmAllStatusEntity.setDeviceRate(deviceOnlineRate);
            farmAllStatusEntity.setDeviceCancel(deviceCancelSum);
            farmAllStatusEntity.setEartagTotal(eartagTotal);
            farmAllStatusEntity.setEartagOnline(eartagOnlineSum);
            farmAllStatusEntity.setEartagRate(eartagOnlineRate);
            farmAllStatusEntity.setUpdateTime(nowTimestamp);
            eartagFarmAllStatusRepo.saveAndFlush(farmAllStatusEntity);
            System.out.println("## 更新的内容="+farmAllStatusEntity);
            //# 同步更新牧场总状态缓存相关字段 #
            cacheService.flashAllStatusCache(farmId,farmAllStatusEntity);
        }
    }

    /**
     * @Method      : cancelEnvtempOfDeviceAllStatus
     * @Description :
     * @Params      : [farmId, todayDateText, todayDate, nowTimestamp]
     * @Return      : void
     *
     * @Author      : ZhuoNing
     * @Date        : 2022/4/1
     * @Time        : 16:26
     */
    public void cancelEnvtempOfDeviceAllStatus(String farmId, String todayDateText, java.sql.Date todayDate, Timestamp nowTimestamp) throws ParseException {
//        System.out.println("## farmId>>"+farmId);
        //{获取处于工作状态的设备数量}
        Integer deviceTotal = getDeviceTotalByFarmIdFromDeviceRegister(farmId);
        System.out.println("## A.deviceTotal>>"+deviceTotal);
        if(deviceTotal == 0){
            log.error("出错:设备总数(deviceTotal)不能为0.");
        }
        //设备在线数量
        Integer deviceOnlineSum = cacheService.getDeviceOnlineSum(farmId,todayDateText);
        System.out.println("# B.设备当前在线数量(deviceOnlineSum)="+deviceOnlineSum);
//        deviceOnlineSum--;
//        System.out.println("# 设备当前在线数量(deviceOnlineSum) 2="+deviceOnlineSum);
        //设备离线数量
        Integer deviceOfflineSum = 0;
        if(deviceTotal > deviceOnlineSum){
            deviceOfflineSum = deviceTotal - deviceOnlineSum;
        }
        System.out.println("## C.deviceOnlineSum>>"+deviceOnlineSum);
        System.out.println("## D.deviceOfflineSum>>"+deviceOfflineSum);
        //设备在线率
        BigDecimal deviceRateBd= new BigDecimal(deviceOnlineSum).divide(new BigDecimal(deviceTotal),1,BigDecimal.ROUND_HALF_UP);
        Float deviceOnlineRate = Float.parseFloat(deviceRateBd.toString());
        System.out.println("## E.deviceOnlineRate>>"+deviceOnlineRate);
        //{获取注销设备数量}
        Integer deviceCancelSum = getDeviceCancelTotalFromDeviceRegister(farmId);
        Integer eartagOnlineSum = 0;
        Float eartagOnlineRate = 0F;
        //耳标总数量
        Integer eartagTotal = eartagService.getEartagRegTotalByFarmId(farmId);
        if(eartagTotal != 0){
            //耳标在线数量
            eartagOnlineSum = eartagService.getEartagOnelineTotal(farmId,todayDateText);
            //耳标在线率
            BigDecimal eartagRateBd= new BigDecimal(eartagOnlineSum).divide(new BigDecimal(eartagTotal),1,BigDecimal.ROUND_HALF_UP);
            eartagOnlineRate = Float.parseFloat(eartagRateBd.toString());
        }
        //
        EartarFarmAllStatusEntity farmAllStatusEntity = eartagFarmAllStatusRepo.getOneByFarmIdAndCreateDate(farmId,todayDateText);
        if(farmAllStatusEntity == null){
            //{新建设备状态记录并更新缓存cache}
            createNewFarmAllStatus(farmId,todayDate,nowTimestamp,deviceTotal,deviceOnlineSum,deviceOfflineSum,deviceCancelSum,deviceOnlineRate,eartagTotal,eartagOnlineSum,eartagOnlineRate);
        }else{
            //计算设备状态并更新缓存cache(在线数+1，离线数-1，计算设备在线率)
            farmAllStatusEntity.setDeviceTotal(deviceTotal);
            farmAllStatusEntity.setDeviceOnline(deviceOnlineSum);
            farmAllStatusEntity.setDeviceOffline(deviceOfflineSum);
            farmAllStatusEntity.setDeviceRate(deviceOnlineRate);
            farmAllStatusEntity.setDeviceCancel(deviceCancelSum);
            farmAllStatusEntity.setEartagTotal(eartagTotal);
            farmAllStatusEntity.setEartagOnline(eartagOnlineSum);
            farmAllStatusEntity.setEartagRate(eartagOnlineRate);
            farmAllStatusEntity.setUpdateTime(nowTimestamp);
            eartagFarmAllStatusRepo.saveAndFlush(farmAllStatusEntity);
            System.out.println("## 更新的内容="+farmAllStatusEntity);
            //# 同步更新牧场总状态缓存相关字段 #
            cacheService.flashAllStatusCache(farmId,farmAllStatusEntity);
        }
    }

    //计算处于不同生存状态的设备数量
    private Integer getDeviceSum(String farmId,Integer liveStatus){
        //# 计算处于工作状态的设备总数（设备注册表）
        List<Object[]> deviceRegList = deviceRegisterRepo.getDeviceCountByFarmIdAndLiveStatus(farmId,liveStatus);
        Object[] deviceRegObj = (Object[]) deviceRegList.get(0);
        //设备总数
        return Integer.parseInt(deviceRegObj[0].toString());
    }

    //# 创建总状态记录 #
    private void createNewFarmAllStatus(String farmId, java.sql.Date todayDate, Timestamp nowTimestamp, Integer deviceTotal, Integer deviceOnlineSum, Integer deviceOfflineSum, Integer deviceCancelSum, Float deviceOnlineRate, Integer eartagTotal, Integer eartagOnlineSum, Float eartagOnlineRate) {
        Integer liveStatus = 1;
        if(getDeviceSum(farmId,liveStatus) == 0){
            log.error("该牧场没有任何处于工作状态的采集器设备.");
        }else{
            //# 计算耳标总数（耳标注册表）
            EartarFarmAllStatusEntity farmAllStatusEntity = new EartarFarmAllStatusEntity();
            farmAllStatusEntity.setDeviceTotal(deviceTotal);
            farmAllStatusEntity.setDeviceOnline(deviceOnlineSum);
            farmAllStatusEntity.setDeviceOffline(deviceOfflineSum);
            farmAllStatusEntity.setDeviceRate(deviceOnlineRate);
            farmAllStatusEntity.setDeviceCancel(deviceCancelSum);
            farmAllStatusEntity.setEartagTotal(eartagTotal);
            farmAllStatusEntity.setEartagOnline(eartagOnlineSum);
            farmAllStatusEntity.setEartagRate(eartagOnlineRate);
            farmAllStatusEntity.setFarmId(farmId);
            farmAllStatusEntity.setUpdateTime(nowTimestamp);
            farmAllStatusEntity.setCreateDate(todayDate);
            eartagFarmAllStatusRepo.saveAndFlush(farmAllStatusEntity);
            //设置总状态缓存
            cacheService.flashAllStatusCache(farmId,farmAllStatusEntity);
        }
    }

    //# 创建总状态记录 #
    private void createNewFarmAllStatus(String farmId,String todayDateText, java.sql.Date todayDate, Timestamp nowTimestamp) {
        Integer liveStatus = 1;
        if(getDeviceSum(farmId,liveStatus) == 0){
            log.error("该牧场没有任何处于工作状态的采集器设备.");
        }else{
            //# 计算耳标总数（耳标注册表）
            EartarFarmAllStatusEntity farmAllStatusEntity = new EartarFarmAllStatusEntity();
            //{获取处于工作状态的设备数量}
            Integer deviceTotal = getDeviceTotalByFarmIdFromDeviceRegister(farmId);
            //设备在线数量
            Integer deviceOnlineSum = cacheService.getDeviceOnlineSum(farmId,todayDateText);
            deviceOnlineSum++;
            //设备离线数量
            Integer deviceOfflineSum = 0;
            if(deviceTotal > deviceOnlineSum){
                deviceOfflineSum = deviceTotal - deviceOnlineSum;
            }
            BigDecimal deviceRateBd= new BigDecimal(deviceOnlineSum).divide(new BigDecimal(deviceTotal),1,BigDecimal.ROUND_HALF_UP);
            //{获取注销设备数量}
            Integer deviceCancelSum = getDeviceCancelTotalFromDeviceRegister(farmId);
            //耳标总数量
            Integer eartagTotal = eartagService.getEartagRegTotalByFarmId(farmId);
            //耳标在线数量
            Integer eartagOnlineSum = eartagService.getEartagOnelineTotal(farmId,todayDateText);
            //耳标在线率
            BigDecimal eartagRateBd= new BigDecimal(eartagOnlineSum).divide(new BigDecimal(eartagTotal),1,BigDecimal.ROUND_HALF_UP);
            farmAllStatusEntity.setDeviceTotal(deviceTotal);
            farmAllStatusEntity.setDeviceOnline(deviceOnlineSum);
            farmAllStatusEntity.setDeviceOffline(deviceOfflineSum);
            farmAllStatusEntity.setDeviceRate(Float.parseFloat(deviceRateBd.toString()));
            farmAllStatusEntity.setDeviceCancel(deviceCancelSum);
            farmAllStatusEntity.setEartagTotal(eartagTotal);
            farmAllStatusEntity.setEartagOnline(eartagOnlineSum);
            farmAllStatusEntity.setEartagRate(Float.parseFloat(eartagRateBd.toString()));
            farmAllStatusEntity.setFarmId(farmId);
            farmAllStatusEntity.setUpdateTime(nowTimestamp);
            farmAllStatusEntity.setCreateDate(todayDate);
            eartagFarmAllStatusRepo.saveAndFlush(farmAllStatusEntity);
            //设置总状态缓存
            cacheService.flashAllStatusCache(farmId,farmAllStatusEntity);
        }
    }

    /**
     * @Method      : updateDeviceOnlineWithEnvtemp
     * @Description : 更新在线统计
     * @Params      :
     * @Return      :
     *
     * @Author      : ZhuoNing
     * @Date        : 2022/3/14
     * @Time        : 14:27
     */
    private void updateDeviceOnlineWithEnvtemp(String farmId, String deviceCode,String envtemp,java.sql.Date todayDate,Timestamp nowTimestamp) throws ParseException {
        String todayDateText = new com.huimv.eartag2.common.utils.DateUtil().getTodayDateText();
        String hashKey = BizConst.DEVICE_REGISTER_PREFIX+deviceCode;
        //计算设备别名
        String alias = (String)cacheService.getHashProperty(hashKey,"deviceAlias");
        //读取在线日期
        EartagDeviceOnlineEntity eartagDeviceOnlineEntity = eartagDeviceOnlineRepo.getDeviceOnline(deviceCode, todayDateText);
        if (eartagDeviceOnlineEntity == null) {
            Integer heartbeatTotal = 0;
            Integer envtempTotal = 1;
            Timestamp heartbeatFirstTime = null;
            Timestamp heartbeatLastTime = null;
            Timestamp envtempFirstTime = nowTimestamp;
            Timestamp envtempLastTime = nowTimestamp;
            Timestamp firstTime = nowTimestamp;
            Timestamp lastTime = nowTimestamp;
            //{计算耳标总数}
//            Integer eartagTotal = eartagService.getEartagRegTotalByFarmId(farmId);
            Integer eartagTotal = 0;
            Integer liveStatus = 1;
            //{新建设备在线统计并创建缓存}
            newDeviceOnline(deviceCode,alias,heartbeatTotal,envtempTotal,eartagTotal,envtemp,todayDate,farmId,heartbeatFirstTime,heartbeatLastTime,envtempFirstTime,envtempLastTime,firstTime,lastTime,liveStatus);
        }else{
            eartagDeviceOnlineEntity.setAlias(alias);
            int envtempTotal = eartagDeviceOnlineEntity.getEnvtempTotal();
            eartagDeviceOnlineEntity.setEnvtempTotal(++envtempTotal);
            eartagDeviceOnlineEntity.setLastEnvtemp(envtemp);
            if(eartagDeviceOnlineEntity.getEnvtempFirstTime() == null){
                eartagDeviceOnlineEntity.setEnvtempFirstTime(nowTimestamp);
            }
            eartagDeviceOnlineEntity.setEnvtempLastTime(nowTimestamp);
            if(eartagDeviceOnlineEntity.getFirstTime() == null){
                eartagDeviceOnlineEntity.setFirstTime(nowTimestamp);
            }
            eartagDeviceOnlineEntity.setLastTime(nowTimestamp);
            eartagDeviceOnlineRepo.saveAndFlush(eartagDeviceOnlineEntity);
            //更新设备在线缓存数据
            cacheService.putDeviceInOnline(deviceCode,eartagDeviceOnlineEntity);
        }
    }

    /**
     * @Method      : newDeviceOnline
     * @Description : 通过心跳创建在线统计记录
     * @Params      : [deviceCode, alias, heartbeatTotal, envtempTotal, eartagTotal, todayDate, farmId, heartbeatFirstTime, heartbeatLastTime, envtempFirstTime, envtempLastTime]
     * @Return      : void
     *
     * @Author      : ZhuoNing
     * @Date        : 2022/3/14
     * @Time        : 21:23
     */
    private void newDeviceOnline(String deviceCode, String alias, Integer heartbeatTotal, Integer envtempTotal, Integer eartagTotal, java.sql.Date todayDate, String farmId, Timestamp heartbeatFirstTime, Timestamp heartbeatLastTime, Timestamp envtempFirstTime, Timestamp envtempLastTime, Timestamp firstTime, Timestamp lastTime, Integer liveStatus) {
        EartagDeviceOnlineEntity newEartagDeviceOnlineEntity = new EartagDeviceOnlineEntity();
        newEartagDeviceOnlineEntity.setDeviceCode(deviceCode);
        newEartagDeviceOnlineEntity.setAlias(alias);
        newEartagDeviceOnlineEntity.setHeartbeatTotal(heartbeatTotal);;
        newEartagDeviceOnlineEntity.setEnvtempTotal(envtempTotal);
        newEartagDeviceOnlineEntity.setEartagTotal(eartagTotal);
        newEartagDeviceOnlineEntity.setAddDate(todayDate);
        newEartagDeviceOnlineEntity.setFarmId(farmId);
        newEartagDeviceOnlineEntity.setHeartbeatFirstTime(heartbeatFirstTime);
        newEartagDeviceOnlineEntity.setHeartbeatLastTime(heartbeatLastTime);
        newEartagDeviceOnlineEntity.setEnvtempFirstTime(envtempFirstTime);
        newEartagDeviceOnlineEntity.setEnvtempLastTime(envtempLastTime);
        newEartagDeviceOnlineEntity.setFirstTime(firstTime);
        newEartagDeviceOnlineEntity.setLastTime(lastTime);
        newEartagDeviceOnlineEntity.setLiveStatus(liveStatus);
        eartagDeviceOnlineRepo.saveAndFlush(newEartagDeviceOnlineEntity);
        //将新建的设备在线数据更新到设备在线缓存
        cacheService.putDeviceInOnline(deviceCode,newEartagDeviceOnlineEntity);
    }

    /**
     * @Method      : newDeviceOnline
     * @Description : 创建设备在线
     * @Params      : [farmId, deviceCode]
     * @Return      : void
     *newDeviceOnline(deviceCode,alias,heartbeatTotal,envtempTotal,temp,todayDate,farmId,heartbeatFirstTime,heartbeatLastTime,nowTimestamp);
     * @Author      : ZhuoNing
     * @Date        : 2022/3/14
     * @Time        : 14:27
     */
    private void newDeviceOnline(String deviceCode, String alias, Integer heartbeatTotal, Integer envtempTotal, Integer eartagTotal, String temp, java.sql.Date todayDate, String farmId, Timestamp heartbeatFirstTime, Timestamp heartbeatLastTime, Timestamp envtempFirstTime, Timestamp envtempLastTime, Timestamp firstTime, Timestamp lastTime, Integer liveStatus) {
        EartagDeviceOnlineEntity newEartagDeviceOnlineEntity = new EartagDeviceOnlineEntity();
        newEartagDeviceOnlineEntity.setDeviceCode(deviceCode);
        newEartagDeviceOnlineEntity.setAlias(alias);
        newEartagDeviceOnlineEntity.setHeartbeatTotal(heartbeatTotal);;
        newEartagDeviceOnlineEntity.setEnvtempTotal(envtempTotal);
        newEartagDeviceOnlineEntity.setEartagTotal(eartagTotal);
        newEartagDeviceOnlineEntity.setLastEnvtemp(temp);
        newEartagDeviceOnlineEntity.setAddDate(todayDate);
        newEartagDeviceOnlineEntity.setFarmId(farmId);
        newEartagDeviceOnlineEntity.setHeartbeatFirstTime(heartbeatFirstTime);
        newEartagDeviceOnlineEntity.setHeartbeatLastTime(heartbeatLastTime);
        newEartagDeviceOnlineEntity.setEnvtempFirstTime(envtempFirstTime);
        newEartagDeviceOnlineEntity.setEnvtempLastTime(envtempLastTime);
        newEartagDeviceOnlineEntity.setFirstTime(firstTime);
        newEartagDeviceOnlineEntity.setLastTime(lastTime);
        newEartagDeviceOnlineEntity.setLiveStatus(liveStatus);
        eartagDeviceOnlineRepo.saveAndFlush(newEartagDeviceOnlineEntity);
        //将新建的设备在线数据更新到设备在线缓存
        cacheService.putDeviceInOnline(deviceCode,newEartagDeviceOnlineEntity);
    }

    //更新总状态表
//    private void updateAllStatus(JSONObject dataJo) {
//    }

    //更新在线统计表
//    private void updateDeviceOnline(JSONObject dataJo) {
//    }


    @Override
    public String getFarmIdByDeviceCode(String deviceCode) {
        EartagDeviceRegisterEntity deviceRegisterEntity = deviceRegisterRepo.getDeviceRegister(deviceCode);
        if(deviceRegisterEntity == null){
            log.error("该采集器设备["+deviceCode+"]的已经不存在。");
            return null;
        }
        return deviceRegisterEntity.getFarmId();
    }

    @Override
    public void countEffectiveDeviceTotal (JSONObject deviceRegisterJo) {
        String farmId = deviceRegisterJo.getString("farmId");
        //{}
        Integer effectiveDeviceTotal = getEffectiveDeviceSum(farmId);
        //{设置设备总数}
        cacheService.putDeviceTotal(effectiveDeviceTotal,farmId);
    }

    @Override
    public void countFarmAllStatus(JSONObject deviceRegisterJo) throws ParseException {
        String farmId = deviceRegisterJo.getString("farmId");
        //{}
//        remoteProcessFarmAllStatus(farmId);
        String todayDateText = new com.huimv.eartag2.common.utils.DateUtil().getTodayDateText();
        java.sql.Date todayDate = new java.sql.Date(new java.util.Date().getTime());
        Timestamp nowTimestamp = new Timestamp(new java.util.Date().getTime());
        //{更新牧场总状态}
        updateEnvtempOfDeviceAllStatus(farmId,todayDateText,todayDate,nowTimestamp);
    }

    @Override
    public void countEffectiveDeviceTotal(Map inMap) {
        String farmId = inMap.get("farmId").toString();
        //{}
        Integer effectiveDeviceTotal = getEffectiveDeviceSum(farmId);
        System.out.println("##<注销设备> effectiveDeviceTotal>>"+effectiveDeviceTotal);
        //{设置设备总数}
        cacheService.putDeviceTotal(effectiveDeviceTotal,farmId);
    }

    @Override
    public void cancelFarmAllStatus(Map map) throws ParseException {
        String farmId = map.get("farmId").toString();
        String todayDateText = new com.huimv.eartag2.common.utils.DateUtil().getTodayDateText();
        java.sql.Date todayDate = new java.sql.Date(new java.util.Date().getTime());
        Timestamp nowTimestamp = new Timestamp(new java.util.Date().getTime());
        //{取消牧场总状态}
        cancelEnvtempOfDeviceAllStatus(farmId,todayDateText,todayDate,nowTimestamp);
    }

    @Override
    public void cancelDeviceOnlineLiveStatus(Map map, String todayDateText) {
        String deviceCode = map.get("deviceCode").toString();
        String farmId = map.get("farmId").toString();
        System.out.println("## <cancelDeviceOnlineLiveStatus> todayDateText="+todayDateText);
        System.out.println("## <cancelDeviceOnlineLiveStatus> deviceCode="+deviceCode);
        EartagDeviceOnlineEntity deviceOnlineEntity = eartagDeviceOnlineRepo.getDeviceOnline(deviceCode,todayDateText);
        System.out.println("## <cancelDeviceOnlineLiveStatus> deviceOnlineEntity="+deviceOnlineEntity);
        if(deviceOnlineEntity == null){
            log.error("该设备当天在线记录不存在.");
            //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        }else{
            Integer liveStatus = 0;
            deviceOnlineEntity.setLiveStatus(liveStatus);
            eartagDeviceOnlineRepo.saveAndFlush(deviceOnlineEntity);
            //更新缓存(从缓存里将该设备从set里取出,同时将该数据表对应的缓存更新)
            cacheService.putDeviceInOnline(deviceCode,deviceOnlineEntity);
            //从在线集合种删除设备编码
            cacheService.deleteDeviceToOnlineSet(farmId,deviceCode,todayDateText);
        }
    }

    @Override
    public void unCancelDeviceOnlineLiveStatus(JSONObject deviceRegisterJo, String todayDateText) {
        String deviceCode = deviceRegisterJo.getString("deviceCode");
        String farmId = deviceRegisterJo.getString("farmId");
        System.out.println("## <unCancelDeviceOnlineLiveStatus> deviceCode="+deviceCode);
        System.out.println("## <unCancelDeviceOnlineLiveStatus> todayDateText="+todayDateText);
        EartagDeviceOnlineEntity deviceOnlineEntity = eartagDeviceOnlineRepo.getDeviceOnline(deviceCode,todayDateText);
        if(deviceOnlineEntity == null){
            log.error("该设备当天在线记录不存在.");
            //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        }else{
            Integer liveStatus = 1;
            deviceOnlineEntity.setLiveStatus(liveStatus);
            eartagDeviceOnlineRepo.saveAndFlush(deviceOnlineEntity);
            //更新缓存(从缓存里将该设备从set里取出,同时将该数据表对应的缓存更新)
            cacheService.putDeviceInOnline(deviceCode,deviceOnlineEntity);
            //从在线集合种删除设备编码
            cacheService.putDeviceToOnlineSet(farmId,deviceCode,todayDateText);
        }
    }

    @Override
    public void remoteProcessFarmAllStatus(String farmId) throws ParseException {
        String todayDateText = new com.huimv.eartag2.common.utils.DateUtil().getTodayDateText();
        java.sql.Date todayDate = new java.sql.Date(new java.util.Date().getTime());
        Timestamp nowTimestamp = new Timestamp(new java.util.Date().getTime());

        RestTemplate restTemplate = new RestTemplate();

        String url = "http://" + serviceFarmAllStatusHost + ":" + serviceFarmAllStatusIp + serviceFarmAllStatusUrl;
        System.out.println("## url>>"+url);

        FarmAllStatusVo farmAllStatusVo = new FarmAllStatusVo();
        farmAllStatusVo.setFarmId(farmId);
        farmAllStatusVo.setTodayDateText(todayDateText);
        farmAllStatusVo.setTodayDate(todayDate);
        farmAllStatusVo.setNowTimestamp(nowTimestamp);
        //{}
        String result = restTemplate.postForObject(url,farmAllStatusVo,String.class);
        System.out.println("result>>"+result);
    }

    //读取有效的设备数量
    public Integer getEffectiveDeviceSum(String farmId){
        List<Object[]> deviceRegisterList = deviceRegisterRepo.getDeviceCountByFarmId(farmId);
        Object[] deviceRegObj = (Object[]) deviceRegisterList.get(0);
        //设备总数
        return Integer.parseInt(deviceRegObj[0].toString());
    }

    /**
     * @Method      : saveDeviceHeartbeat
     * @Description : 保存设备心跳数据
     * @Params      : [dataJo]
     * @Return      : void
     *
     * @Author      : ZhuoNing
     * @Date        : 2022/2/15
     * @Time        : 14:39
     */
    @Override
    public void saveDeviceHeartbeat(JSONObject dataJo, java.sql.Date todayDate) {
        String deviceCode = dataJo.getString("device");
        EartagHeartbeatEntity addHeartbeatEntity = new EartagHeartbeatEntity();
        addHeartbeatEntity.setDeviceCode(deviceCode);
        addHeartbeatEntity.setAddTime(new Timestamp(new Date().getTime()));
        addHeartbeatEntity.setCreateDate(todayDate);
        heartbeatRepo.saveAndFlush(addHeartbeatEntity);
    }

    /**
     * @Method      : saveDeviceTemp
     * @Description : 保存设备温度数据
     * @Params      : [dataJo]
     * @Return      : void
     *
     * @Author      : ZhuoNing
     * @Date        : 2022/2/15
     * @Time        : 14:39
     */
    @Override
    public void saveDeviceTemp(JSONObject dataJo, java.sql.Date todayDate, Timestamp nowTimestamp) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        MathUtil mathUtil = new MathUtil();
        String device = dataJo.getString("device");
        String askTime = dateUtil.rebuildDateTime(dataJo.getString("askTime"));
        String temp = dataJo.getString("temp");
        EartagEnvEntity envEntity = new EartagEnvEntity();
        envEntity.setDeviceCode(device);
        envEntity.setTemp1(Integer.parseInt(temp));
        envEntity.setTemp2(Float.parseFloat(mathUtil.countEnvtemp(temp,2)));
        envEntity.setAddTime(nowTimestamp);
        envEntity.setAskTime(new Timestamp(dateUtil.parseDateTime(askTime).getTime()));
        envEntity.setCreateDate(todayDate);
        envEntity.setAskDate(new java.sql.Date(dateUtil.parseDate(dateUtil.getAskDate(dataJo.getString("askTime"))).getTime()));
        envRepo.saveAndFlush(envEntity);
    }

    /**
     * @Method      : saveRawdata
     * @Description : 保存原始数据
     * @Params      : [askText]
     * @Return      : void
     *
     * @Author      : ZhuoNing
     * @Date        : 2022/2/9
     * @Time        : 13:38
     */
    @Override
    public void saveRawdata(String askText) throws ParseException {
//        log.info("#开始解析Ask请求数据>>" + askText.trim().replace("\n", ""));
        String[] dataArray = askText.split("\\+");
        String cmdHeader = dataArray[0];
        Map rawMap = new HashMap();
        if (cmdHeader.trim().equalsIgnoreCase("hm")) {
            //采集器原始数据
            String device = dataArray[1];
            String cmd = dataArray[2];
            rawMap.put("cmdHeader", cmdHeader);
            rawMap.put("device", device);
            rawMap.put("cmd", cmd);
            rawMap.put("ask", askText);
        } else if (cmdHeader.trim().equalsIgnoreCase("zj")) {
            //采集器原始数据
            String device = dataArray[1];
            rawMap.put("cmdHeader", cmdHeader);
            rawMap.put("device", device);
            rawMap.put("cmd", "");
            rawMap.put("ask", askText);
            rawMap.put("addTime", dateUtil.getTodayMissionText());
        } else {
            //采集器原始数据
            String device = dataArray[1];
            rawMap.put("cmdHeader", cmdHeader);
            rawMap.put("device", device);
            rawMap.put("cmd", "");
            rawMap.put("ask", askText);
        }
        //保存原始数据
        //类型、设备、命令、数据内容、日期
        EartagRawEntity addRawEntity = new EartagRawEntity();
        addRawEntity.setCmdHeader(rawMap.get("cmdHeader").toString());
        addRawEntity.setDevice(rawMap.get("device").toString());
        addRawEntity.setCmd(rawMap.get("cmd").toString());
        addRawEntity.setAsk(rawMap.get("ask").toString());
        addRawEntity.setAddTime(new Timestamp(new Date().getTime()));
        rawRepo.saveAndFlush(addRawEntity);
    }

}
