package com.huimv.eartag2.controller;

import com.alibaba.fastjson.JSONObject;
import com.huimv.eartag2.service.IDeviceService;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@RestController
public class SendMessageController {
    @Autowired
    RabbitTemplate rabbitTemplate;  //使用RabbitTemplate,这提供了接收/发送等等方法
    @Autowired
    private IDeviceService deviceService;

    @GetMapping("/sendDirectMessage")
    public String sendDirectMessage() {
        String messageId = String.valueOf(java.util.UUID.randomUUID());
        String messageData = "test message, hello!";
        String createTime = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        Map<String,Object> map=new HashMap<>();
        map.put("messageId",messageId);
        map.put("messageData",messageData);
        map.put("createTime",createTime);
        //将消息携带绑定键值：TestDirectRouting 发送到交换机TestDirectExchange
        rabbitTemplate.convertAndSend("TestDirectExchange", "TestDirectRouting", map);
        return "ok";
    }

    @GetMapping("/sendTopicMessage1")
    public String sendTopicMessage1() {
        String messageId = String.valueOf(UUID.randomUUID());
        String messageData = "message: M A N ";
        String createTime = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        Map<String, Object> manMap = new HashMap<>();
        manMap.put("messageId", messageId);
        manMap.put("messageData", messageData);
        manMap.put("createTime", createTime);
        rabbitTemplate.convertAndSend("topicExchange", "topic.man", manMap);
        return "ok";
    }

    @GetMapping("/sendTopicMessage2")
    public String sendTopicMessage2() {
        String messageId = String.valueOf(UUID.randomUUID());
        String messageData = "message: woman is all ";
        String createTime = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        Map<String, Object> womanMap = new HashMap<>();
        womanMap.put("messageId", messageId);
        womanMap.put("messageData", messageData);
        womanMap.put("createTime", createTime);
        rabbitTemplate.convertAndSend("topicExchange", "topic.woman", womanMap);
        return "ok";
    }

    @GetMapping("/sendFanoutMessage")
    public String sendFanoutMessage() {
        String messageId = String.valueOf(UUID.randomUUID());
        String messageData = "message: testFanoutMessage ";
        String createTime = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        Map<String, Object> map = new HashMap<>();
        map.put("messageId", messageId);
        map.put("messageData", messageData);
        map.put("createTime", createTime);
        rabbitTemplate.convertAndSend("fanoutExchange", null, map);
        return "ok";
    }

    @GetMapping("/TestMessageAck")
    public String TestMessageAck() {
        String messageId = String.valueOf(UUID.randomUUID());
        String messageData = "message: non-existent-exchange test message ";
        String createTime = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        Map<String, Object> map = new HashMap<>();
        map.put("messageId", messageId);
        map.put("messageData", messageData);
        map.put("createTime", createTime);
        rabbitTemplate.convertAndSend("non-existent-exchange", "TestDirectRouting", map);
        return "ok";
    }

    @GetMapping("/TestMessageAck2")
    public String TestMessageAck2() {
        String messageId = String.valueOf(UUID.randomUUID());
        String messageData = "message: lonelyDirectExchange test message ";
        String createTime = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        Map<String, Object> map = new HashMap<>();
        map.put("messageId", messageId);
        map.put("messageData", messageData);
        map.put("createTime", createTime);
        rabbitTemplate.convertAndSend("lonelyDirectExchange", "TestDirectRouting", map);
        return "ok";
    }

    //topic()
    @GetMapping("/testEartag")
    public String testEartag() {
        String messageId = String.valueOf(UUID.randomUUID());
        String messageData = "message: topic1 ";
        String createTime = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        Map<String, Object> map = new HashMap<>();
        map.put("messageId", messageId);
        map.put("messageData", messageData);
        map.put("createTime", createTime);

        JSONObject dataJo = new JSONObject();
        dataJo.put("name","tony");
        dataJo.put("age","25");
        Map<String,Object> deviceMap = new HashMap<>();
        deviceMap.put("textData",dataJo.toString());
        rabbitTemplate.convertAndSend("deviceTopicExchange", "topic.device", "hello.");
        return "ok";
    }

    @GetMapping("/testEartag2")
    public String testEartag2() {
        JSONObject dataJo = new JSONObject();
        dataJo.put("name","tony");
        dataJo.put("age","25");
        rabbitTemplate.convertAndSend("deviceTopicExchange", "topic.device", dataJo);
        return "ok";
    }

    @GetMapping("/testRegister")
    public String testRegister() {
        JSONObject dataJo = new JSONObject();
        dataJo.put("name","tony");
        dataJo.put("age","25");
        rabbitTemplate.convertAndSend("deviceTopicExchange", "topic.device.register", dataJo);
        return "ok";
    }

    @GetMapping("/testRegister2")
    public String testRegister2() {
        Map map = new HashMap();
        map.put("name","tony");
        map.put("age","25");
        rabbitTemplate.convertAndSend("deviceTopicExchange", "topic.device.register", map);
        return "ok";
    }

    @GetMapping("/testRawdata")
    public String testRawdata() {
        Map map = new HashMap();
        map.put("name","tony");
        map.put("age","25");
        rabbitTemplate.convertAndSend("rawdataExchange", "topic.rawdata", map);
        return "ok";
    }

    @GetMapping("/getDeviceCodeByChipId")
    public String getDeviceCodeByChipId(String chipId) {
        String deviceCode = deviceService.getDeviceCodeByChipId(chipId);
        System.out.println(">>deviceCode>>"+deviceCode);
        return deviceCode;
    }
}
