package com.huimv.eartag2.service.impl;

import com.huimv.eartag2.common.dao.entity.EartagDeviceRegisterEntity;
import com.huimv.eartag2.common.dao.entity.EartagRawEntity;
import com.huimv.eartag2.common.dao.repo.EartagDeviceRegisterRepo;
import com.huimv.eartag2.common.dao.repo.EartagRawRepo;
import com.huimv.eartag2.common.utils.DateUtil;
import com.huimv.eartag2.service.ICacheService;
import com.huimv.eartag2.service.IDeviceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.Date;
import java.sql.Timestamp;
import java.text.ParseException;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
@Slf4j
public class DeviceServiceImpl implements IDeviceService {
    @Autowired
    RabbitTemplate rabbitTemplate;  //使用RabbitTemplate,这提供了接收/发送等等方法
    @Autowired
    private EartagRawRepo eartagRawRepo;
    @Autowired
    private EartagDeviceRegisterRepo deviceRegisterRepo;
    @Autowired
    private ICacheService cacheService;

    //处理采集器设备信息
    @Override
    public void handleDeviceInfo() {
        //--处理注册表数据
        handleEartagRegister();
        //--处理在线记录数据
        handleEartagOnline();
        //--处理状态数据
        handleEartagAllState();
    }

    @Override
    public void saveRawData(String askText) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        EartagRawEntity eartagRawEntity = new EartagRawEntity();
        eartagRawEntity.setAsk(askText);
        eartagRawEntity.setAddTime(dateUtil.getTimestamp());
        eartagRawEntity.setCreateDate(new Date(dateUtil.getTodayDate().getTime()));
        eartagRawRepo.saveAndFlush(eartagRawEntity);
    }

    @Override
    public String getDeviceCodeByChipId(String chipId) {
        //{}
        Object deviceCodeObj = cacheService.getDeviceCodeByChipId(chipId);
        if(deviceCodeObj != null){
            return (String)deviceCodeObj;
        }else{
//            return null;
            //{从数据库读取设备编码返回}
            return getDeviceCodeByChipIdFromMySQL(chipId);
        }
    }

    //从数据库读取deviceCode
    private String getDeviceCodeByChipIdFromMySQL(String chipId) {
        //{}
        EartagDeviceRegisterEntity deviceRegisterEntity = deviceRegisterRepo.getByChipId(chipId);
        if(deviceRegisterEntity == null){
            log.error("该芯片id不存在.");
            return null;
        }else{
            String deviceCode = deviceRegisterEntity.getDeviceCode();
            cacheService.setDeviceCodeByChipId(chipId,deviceCode);
            return deviceCode;
        }
    }

    //--处理状态数据
    private void handleEartagAllState() {

    }

    //--处理在线记录数据
    private void handleEartagOnline() {

    }

    //--处理注册表数据
    private void handleEartagRegister() {

    }
}
