package com.huimv.eartag2.manage.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.huimv.eartag2.common.dao.entity.EartagDeviceRegisterEntity;
import com.huimv.eartag2.common.dao.entity.EartarFarmAllStatusEntity;
import com.huimv.eartag2.common.utils.BizConst;
import com.huimv.eartag2.manage.service.ICacheService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
@Slf4j
public class CacheServiceImpl implements ICacheService {
    @Autowired
    private RedisTemplate redisTemplate;
    @Value("${cache.chipIdExpire}")
    private Integer chipIdExpire;

    @Override
    public Map getFarmAllStatusByFarmId(String farmId) {
        String hashKey = BizConst.FARM_ALL_STATUS_PREFIX+farmId;
        return redisTemplate.opsForHash().entries(hashKey);
    }

    @Override
    public int getDeviceOnlineTotal(String farmId, String todayDateText) {
        String setKey = BizConst.SET_FARM_DEVICE_ONLINE_PREFIX+farmId+"_"+todayDateText;
        Set<String> resultSet = redisTemplate.opsForSet().members(setKey);
        return resultSet.size();
    }

    @Override
    public int getEartagOnlineTotal(String farmId, String todayDateText) {
        String setKey = BizConst.SET_FARM_EARMARK_ONLINE_PREFIX+farmId+"_"+todayDateText;
        Set<String> resultSet = redisTemplate.opsForSet().members(setKey);
        return resultSet.size();
    }

    @Override
    public void putFarmAllStatusCache(String farmId, EartarFarmAllStatusEntity farmAllStatusEntity) {
//        Map allStatusMap = new HashMap();
////        allStatusMap.put("id",farmAllStatusEntity.getId());
//        allStatusMap.put("deviceTotal",farmAllStatusEntity.getDeviceTotal());
//        allStatusMap.put("deviceOnline",farmAllStatusEntity.getDeviceOnline());
//        allStatusMap.put("deviceOffline",farmAllStatusEntity.getDeviceOffline());
//        allStatusMap.put("deviceRate",farmAllStatusEntity.getDeviceRate());
//        allStatusMap.put("deviceCancel",farmAllStatusEntity.getDeviceCancel());
//        allStatusMap.put("eartagTotal",farmAllStatusEntity.getEartagTotal());
//        allStatusMap.put("eartagOnline",farmAllStatusEntity.getEartagOnline());
//        allStatusMap.put("farmId",farmId);
////        allStatusMap.put("updateTime",duObj.formatDateTime(duObj.getTodayText()));
////        allStatusMap.put("createDate",duObj.getTodayDate());
//        allStatusMap.put("updateTime",farmAllStatusEntity.getUpdateTime());
//        allStatusMap.put("createDate",farmAllStatusEntity.getCreateDate());
        Map allStatusMap = JSON.parseObject(JSONObject.toJSONString(farmAllStatusEntity),Map.class);
        redisTemplate.opsForHash().putAll(BizConst.FARM_ALL_STATUS_PREFIX+farmId, allStatusMap);
    }

    @Override
    public boolean isExistDeviceCode(String deviceCode) {
        String setKey = BizConst.SET_ALL_DEVICE_PREFIX;
//        log.info("## setKey>>"+setKey+", deviceCode>>"+deviceCode);
        return redisTemplate.opsForSet().isMember(setKey, deviceCode);
    }

    @Override
    public int isExistAllDeviceSet() {
        String setKey = BizConst.SET_ALL_DEVICE_PREFIX;
        Set<String> resultSet = redisTemplate.opsForSet().members(setKey);
        return resultSet.size();
    }

    @Override
    public void putAllDeviceSet(List list) {
        String setKey = BizConst.SET_ALL_DEVICE_PREFIX;
        for(int a=0;a<list.size();a++){
            redisTemplate.opsForSet().add(setKey,list.get(a).toString());
        }
        //验证
//        Set<String> resultSet = redisTemplate.opsForSet().members("setKey");
//        System.out.println("# resultSet.toString()>>"+resultSet.toString());
    }

    @Override
    public void putAllDeviceSet(String deviceCode){
        String setKey = BizConst.SET_ALL_DEVICE_PREFIX;
        redisTemplate.opsForSet().add(setKey,deviceCode);
    }

    @Override
    public void putChipIdToCache(String chipId, String deviceCode) {
        System.out.println("## <putChipIdToCache> chipId>>"+chipId);
        System.out.println("## <putChipIdToCache> deviceCode>>"+deviceCode);

        String key = BizConst.CHIP_DEVICE_PREFIX;
        String hashKey = chipId;
        //
        redisTemplate.opsForHash().put(key,hashKey,deviceCode);
//        redisTemplate.expire(key,chipIdExpire , TimeUnit.HOURS);
    }

    @Override
    public void putDeviceRegister(String deviceCode, EartagDeviceRegisterEntity deviceRegisterEntity) {
        String key = BizConst.DEVICE_REGISTER_PREFIX+deviceCode;
        System.out.println("## <putDeviceRegister>.key>>"+key);
        Map map = JSON.parseObject(JSONObject.toJSONString(deviceRegisterEntity),Map.class);
        redisTemplate.opsForHash().putAll(key,map);
    }

    @Override
    public void deleteDeviceRegister(String deviceCode) {
        String key = BizConst.DEVICE_REGISTER_PREFIX+deviceCode;
//        System.out.println(">> <deleteDeviceRegister> key>>"+key);
        Object obj = redisTemplate.opsForHash().entries(key);
        if(obj == null){
            log.error("##Error 该key["+key+"]的缓存不存在.");
        }else{
            redisTemplate.delete(key);
//            System.out.println(">> end <deleteDeviceRegister> key>>"+key);
        }
    }

    @Override
    public void putDeviceTotal_(Integer effectiveDeviceTotal, String farmId) {
        String valueKey = BizConst.FARM_DEVICE_TOTAL_PREFIX+farmId;
        redisTemplate.opsForValue().set(valueKey,effectiveDeviceTotal);
        // TEST
//        Object obj = redisTemplate.opsForValue().get(valueKey);
//        System.out.println("TEST value>>"+obj.toString());
    }

    @Override
    public void deleteDeviceCodeSetCache(String deviceCode) {
        String setKey = BizConst.SET_ALL_DEVICE_PREFIX;
//        System.out.println(">> <deleteDeviceCodeSetCache> setKey "+setKey);
//        redisTemplate.opsForSet().add(setKey,deviceCode);
        redisTemplate.opsForSet().remove(setKey, deviceCode);
//        System.out.println(">> end <deleteDeviceCodeSetCache> setKey "+setKey);
    }

    @Override
    public void deleteChipIdHashCache(String chipId) {
        String key = BizConst.CHIP_DEVICE_PREFIX;
        String hashKey = chipId;
//        System.out.println(">> <deleteChipIdHashCache> key>>"+key+",hashKey>>"+hashKey);
        Object obj = redisTemplate.opsForHash().get(key, hashKey);
//        System.out.println(">> obj>>"+obj);
        if(obj == null){
            log.error("##Error 该key>>"+key+",hashKey>>"+hashKey+"的缓存不存在.");
        }else{
            redisTemplate.opsForHash().delete(key,hashKey);
        }

//        Object obj = redisTemplate.opsForHash().entries(key);
//        System.out.println(">> obj>>"+obj);
//        if(obj == null){
//            log.error("##Error 该key["+key+"]的缓存不存在.");
//        }else{
//            redisTemplate.delete(key);
//        }
        System.out.println(">> end <deleteChipIdHashCache> key "+key);
    }

    @Override
    public void getDeviceCodeSetCache() {
        String setKey = BizConst.SET_ALL_DEVICE_PREFIX;
        Set<String> resultSet = redisTemplate.opsForSet().members(setKey);
        System.out.println("## resultSet>>"+resultSet);
    }

    @Override
    public void deleteOnlineDeviceSet(String farmId, String deviceCode, String todayDateText) {
        String setKey = BizConst.SET_FARM_DEVICE_ONLINE_PREFIX + farmId+":"+todayDateText;
        redisTemplate.opsForSet().remove(setKey,deviceCode);
    }

    @Override
    public void putOnlineDeviceSet(String farmId, String deviceCode, String todayDateText) {
        String setKey = BizConst.SET_FARM_DEVICE_ONLINE_PREFIX + farmId+":"+todayDateText;
        redisTemplate.opsForSet().add(setKey,deviceCode);
    }
}
