
package com.huimv.eartag2.api.schedule;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.huimv.eartag2.api.mapper.*;
import com.huimv.eartag2.api.pojo.*;
import com.huimv.eartag2.api.service.EartagDeviceRegisterService;
import com.huimv.eartag2.api.service.IEartagService;
import com.huimv.eartag2.api.service.SysBaseConfigService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import java.io.IOException;
import java.math.BigDecimal;
import java.sql.Timestamp;
import java.text.ParseException;
import java.util.Date;
import java.util.List;


/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/

@Configuration      //1.主要用于标记配置类，兼备Component的效果。
@EnableScheduling   // 2.开启定时任务
public class EartagTask {
    @Autowired
    private IEartagService eartagService;
    @Autowired
    private SysBaseConfigService baseConfigService;
    @Autowired
    private EartagDeviceRegisterService deviceRegisterService;
    @Autowired
    private EartagResetMapper eartagResetMapper;
    @Autowired
    private EartagEartagRegister2Mapper eartagEartagRegister2Mapper;
    @Autowired
    private BaseFarmerMapper baseFarmerMapper;
    @Autowired
    private EartagDeviceRegisterMapper eartagDeviceRegisterMapper;
    @Autowired
    private FarmerAssetMapper farmerAssetMapper;
    @Value("${farmer.pig.price}")
    private String pigPrice;
    @Autowired
    private FarmerDayageWeightMapper farmerDayageWeightMapper;


/**
     * @Method : countEartagOnlineTimes
     * @Description : 判断耳标离线状态
     * @Params : []
     * @Return : void
     * @Author : ZhuoNing
     * @Date : 2022/8/20
     * @Time : 19:35
     */

    // 10分钟
    @Scheduled(cron = "0 0/10 * * * ? ")
    private void countEartagOnlineTimes() throws IOException, ParseException {
        Timestamp nowTimestamp = new Timestamp(new java.util.Date().getTime());
//        System.out.println("时间戳="+nowTimestamp);

//        ExecutorService service = Executors.newFixedThreadPool(5);
//        ExecutorService service = Executors.newSingleThreadExecutor();
//        RunnableTaskEartagOnlineStatus task1 = new RunnableTaskEartagOnlineStatus();
//        service.execute(task1);
        //更新耳标在线状态
        eartagService.updateEartagLiveStatusAndCount();
    }



/**
     * @Method : updateDevice
     * @Description : 判断设备离线状态
     * @Params : []
     * @Return : void
     * @Author : ZhuoNing
     * @Date : 2022/8/20
     * @Time : 19:36
     */

    // 5分钟
    @Scheduled(cron = "0 0/5 * * * ? ")
    private void updateDevice() {
        List<SysBaseConfig> baseConfig = baseConfigService.list(new QueryWrapper<SysBaseConfig>().groupBy("row_id"));


        for (SysBaseConfig sysBaseConfig : baseConfig) {
            String rowId = sysBaseConfig.getRowId();
            SysBaseConfig one = baseConfigService.getOne(new QueryWrapper<SysBaseConfig>().eq("row_id", rowId).eq("config_key", "eartagOfflineTime"));

            String configValueStr = one.getConfigValue();
            if (StringUtils.isNotBlank(configValueStr)) {
                int configValueInt = Integer.parseInt(configValueStr);
                DateTime dateTime = DateUtil.offsetMinute(new Date(), -configValueInt);

                UpdateWrapper<EartagDeviceRegister> wrapper = new UpdateWrapper<>();
                wrapper.ge("last_time", dateTime);
                EartagDeviceRegister deviceRegister = new EartagDeviceRegister();
                //未离线
                deviceRegister.setActiveStatus(1);
                deviceRegister.setDeviceStatus(1);
                deviceRegisterService.update(deviceRegister, wrapper);

                //离线
                deviceRegister.setActiveStatus(2);
                deviceRegister.setDeviceStatus(0);
                wrapper.clear();
                wrapper.le("last_time", dateTime);
                deviceRegisterService.update(deviceRegister, wrapper);
            } else {
                System.out.println("未设置定时时长");
            }

            System.out.println("测试数据");
        }

    }

    @Scheduled(cron = "0 0 0/6 * * ? ")
    private void countAssetInEveryMonth() throws ParseException {
        //--读取所有农户和采集器
        List<BaseFarmer> baseFarmerList = baseFarmerMapper.getFarmer();
        //--用采集器读取所有数据计算可售和不可售资源
        for (BaseFarmer baseFarmer : baseFarmerList) {
            //--计算每个农户的生物资产
            _countAssetByFarmer(baseFarmer);
        }
    }

    //--
    private void _countAssetByFarmer(BaseFarmer baseFarmer) throws ParseException {
        //------------------------------------------------------------------------
        // 读取采集器.
        // 根据采集器读取所有耳标.
        // 对读取耳标进行计算可售和非可售，并归到当月.
        //------------------------------------------------------------------------

        com.huimv.eartag2.common.utils.DateUtil dateUtil = new com.huimv.eartag2.common.utils.DateUtil();
        Integer farmerId = baseFarmer.getId();
        //--读取采集器
        String deviceCode = eartagDeviceRegisterMapper.getDeviceIdByFarmerId(String.valueOf(farmerId));
        java.sql.Date todayDate = new java.sql.Date(new java.util.Date().getTime());
        //--读取注册耳标
        List<EartagEartagRegister2> eartagEartagRegister2List = eartagEartagRegister2Mapper.getEartagByDeviceCode(deviceCode);
        String year = dateUtil.getThisYear();
        String month = dateUtil.getThisMonth();
        if(eartagEartagRegister2List.size()>0){
            //--
            List<FarmerDayageWeight> farmerDayageWeightList = farmerDayageWeightMapper.getAllDayageWeight();
            BigDecimal availTotalBd = new BigDecimal(0);
            BigDecimal unavailTotalBd = new BigDecimal(0);
            for (EartagEartagRegister2 eartagEartagRegister2 : eartagEartagRegister2List) {
                long diff = todayDate.getTime() - eartagEartagRegister2.getCreateDate().getTime();
                long days = diff / (1000 * 60 * 60 * 24);
                if (days < 180) {
                    for (FarmerDayageWeight farmerDayageWeight : farmerDayageWeightList) {
                        if (farmerDayageWeight.getStartDayage() <= days && days <= farmerDayageWeight.getEndDayage()) {
                            availTotalBd = availTotalBd.add(farmerDayageWeight.getWeight().multiply(new BigDecimal(pigPrice)).setScale(2, BigDecimal.ROUND_HALF_UP));
                            break;
                        }
                    }
                } else if (180 <= days && days < 210) {
                    for (FarmerDayageWeight farmerDayageWeight : farmerDayageWeightList) {
                        if (farmerDayageWeight.getStartDayage() <= days && days <= farmerDayageWeight.getEndDayage()) {
                            unavailTotalBd = unavailTotalBd.add(farmerDayageWeight.getWeight().multiply(new BigDecimal(pigPrice)).setScale(2, BigDecimal.ROUND_HALF_UP));
                            break;
                        }
                    }
                }
            }
            //--保存生物资产
            saveFarmerAsset(farmerId, year, month, availTotalBd, unavailTotalBd);
        }else{
            saveFarmerAsset(farmerId, year, month, new BigDecimal(0), new BigDecimal(0));
        }
    }

    //--保存生物资产
    private void saveFarmerAsset(Integer farmerId, String year, String month, BigDecimal availTotalBd, BigDecimal unavailTotalBd) {
        //--
        FarmerAsset farmerAsset = farmerAssetMapper.getAssetByFarmerIdAndYearAndMonth(farmerId, year, month);
        if (farmerAsset == null) {
            FarmerAsset farmerAssetEntity = new FarmerAsset();
            farmerAssetEntity.setFarmCode(String.valueOf(farmerId));
            farmerAssetEntity.setYear(Integer.parseInt(year));
            farmerAssetEntity.setMonth(Integer.parseInt(month));
            farmerAssetEntity.setAvailAsset(availTotalBd);
            farmerAssetEntity.setUnavailAsset(unavailTotalBd);
            farmerAssetMapper.insert(farmerAssetEntity);
        } else {
            farmerAsset.setAvailAsset(availTotalBd);
            farmerAsset.setUnavailAsset(unavailTotalBd);
            farmerAssetMapper.updateById(farmerAsset);
        }
    }
}

