package com.huimv.eartag2.api.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.eartag2.common.utils.DateUtil;
import com.huimv.eartag2.common.utils.Result;
import com.huimv.eartag2.common.utils.ResultCode;
import com.huimv.eartag2.api.mapper.*;
import com.huimv.eartag2.api.pojo.*;
import com.huimv.eartag2.api.service.EartagAbnormalAlarmService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.util.*;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zn
 * @since 2022-08-30
 */
@Service
public class EartagAbnormalAlarmServiceImpl extends ServiceImpl<EartagAbnormalAlarmMapper, EartagAbnormalAlarm> implements EartagAbnormalAlarmService {
    @Autowired
    private EartagAbnormalAlarmMapper eartagAbnormalAlarmMapper;
    @Autowired
    private EartagEartagRegister2Mapper eartagEartagRegister2Mapper;
    @Autowired
    private EartagDeviceRegisterMapper eartagDeviceRegisterMapper;
    @Autowired
    private BaseFarmerMapper baseFarmerMapper;
    @Autowired
    private BaseFarmMapper baseFarmMapper;

    @Override
    public Result getEarTemWarning(Map<String, Object> paramsMap) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        String farmCode = paramsMap.get("areaId")+"";
        java.sql.Date todayDate = new java.sql.Date(new java.util.Date().getTime());
        QueryWrapper<EartagAbnormalAlarm> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code",farmCode);
        queryWrapper.eq("alarm_from","1").or().eq("alarm_from","2");
        queryWrapper.eq("alarm_type","2");
        queryWrapper.eq("add_date",todayDate);

        queryWrapper.orderByDesc("add_time");
        queryWrapper.last("limit 50");

        List<EartagAbnormalAlarm> eartagAbnormalAlarmList =  eartagAbnormalAlarmMapper.selectList(queryWrapper);
        JSONArray newJa = new JSONArray();
        for(EartagAbnormalAlarm eartagAbnormalAlarm:eartagAbnormalAlarmList){
            JSONObject newJo = new JSONObject();
            newJa.add(newJo);
            newJo.put("addTime",dateUtil.formatDatetimeText(eartagAbnormalAlarm.getAddTime()));
            newJo.put("temp",eartagAbnormalAlarm.getTempValue());
            newJo.put("content",eartagAbnormalAlarm.getContent());
        }
        JSONObject resultJo = new JSONObject();
        resultJo.put("total",eartagAbnormalAlarmList.size());
        resultJo.put("content",newJa);
        return new Result(ResultCode.SUCCESS,resultJo);
    }

    @Override
    public Result getNewPig(Map<String, String> paramsMap) {
        String areaId = paramsMap.get("areaId");
        QueryWrapper<EartagEartagRegister2> wrapper = new QueryWrapper<>();

        List endList = new ArrayList();
        Date now = new Date();

        for (int i = 0 ; i<6 ;i++){
            DateTime beginOfMonth = cn.hutool.core.date.DateUtil.beginOfMonth(now);
            wrapper.clear();
            wrapper.eq("farm_id",areaId);

            beginOfMonth = cn.hutool.core.date.DateUtil.offsetMonth(beginOfMonth,-i);
            DateTime endOfMonth = cn.hutool.core.date.DateUtil.endOfMonth(beginOfMonth);
            wrapper.between("first_time",beginOfMonth,endOfMonth);
            Long release = eartagEartagRegister2Mapper.selectCount(wrapper);
            wrapper.clear();
            wrapper.eq("farm_id",areaId);
            wrapper.between("first_time",cn.hutool.core.date.DateUtil.offsetMonth(beginOfMonth,-6),cn.hutool.core.date.DateUtil.offsetMonth(endOfMonth,-6));
            Long slaughter = eartagEartagRegister2Mapper.selectCount(wrapper);

            Map map  = new HashMap();
            map.put("release",release);
            map.put("slaughter",slaughter);
            map.put("month",(cn.hutool.core.date.DateUtil.month(beginOfMonth) +1) +"月");
            endList.add(map);


        }

        return new Result(ResultCode.SUCCESS,endList);
    }

    @Override
    public Result listFarmer() {
        List<BaseFarmer> farmers = baseFarmerMapper.selectList(null);
        List endList = new ArrayList<>();
        for (BaseFarmer farmer : farmers) {
            BaseFarm baseFarm = baseFarmMapper.selectOne(new QueryWrapper<BaseFarm>().eq("farm_code", farmer.getFarmCode()));

            Map map  = new HashMap();
            map.put("farmCode",farmer.getFarmCode());
            map.put("farmerName",farmer.getFarmer());
            map.put("id",farmer.getId());
            if(ObjectUtil.isNotEmpty(baseFarm)){

                map.put("areaCode",baseFarm.getAreaCode());
                map.put("farmName",baseFarm.getFarmName());
            }else {
                map.put("areaCode","");
                map.put("farmName","");
            }


            endList.add(map);

        }
        return new Result(ResultCode.SUCCESS,endList);
    }

    @Override
    public Result countAll(Map<String, Object> paramsMap) {
        String farmCode = (String)paramsMap.get("farmId");
        HashMap<String,Long> endMap = new HashMap();
        Long countFarmer = baseFarmerMapper.selectCount(new QueryWrapper<BaseFarmer>().eq("farm_code", farmCode));;
        Long countPig = eartagEartagRegister2Mapper.selectCount(new QueryWrapper<EartagEartagRegister2>().eq("farm_id", farmCode).eq("live_status",1).like("earmark","31111112222"));
        Long countDevice = eartagDeviceRegisterMapper.selectCount(new QueryWrapper<EartagDeviceRegister>().eq("farm_id", farmCode));
        java.sql.Date todayDate = new java.sql.Date(new java.util.Date().getTime());
        Long countWarning = eartagAbnormalAlarmMapper.selectCount(new QueryWrapper<EartagAbnormalAlarm>().eq("farm_code",farmCode).eq("alarm_type","2").eq("add_date",todayDate));


        endMap.put("countFarmer",countFarmer);
        endMap.put("countPig",countPig);
        endMap.put("countDevice",countDevice);
        endMap.put("countWarning",countWarning);
        return new Result(ResultCode.SUCCESS,endMap);
    }

    @Override
    public Result pigMonitor(Map<String, Object> paramsMap) {
        String farmCode = (String)paramsMap.get("farmId");
        List<EartagEartagRegister2> eartagEartagRegister2s = eartagEartagRegister2Mapper.selectList(new QueryWrapper<EartagEartagRegister2>().eq("farm_id", farmCode).orderByDesc("last_time").last("limit 5"));
        return new Result(ResultCode.SUCCESS,eartagEartagRegister2s);
    }

    @Override
    public Result getFarmerAndPig(Map<String, Object> paramsMap) {
        List<BaseFarmer> farmers = baseFarmerMapper.selectList(new QueryWrapper<BaseFarmer>().eq("farm_code", "330727001"));

        List endList = new ArrayList();
        for (BaseFarmer farmer : farmers) {
            Map map = new HashMap();
            List<EartagDeviceRegister> eartagDeviceRegisters = eartagDeviceRegisterMapper.selectList(new QueryWrapper<EartagDeviceRegister>().eq("farmer_id", farmer.getId()).eq("farm_id","330727001"));
            map.put( "data",new ArrayList<>());
            map.put("id",farmer.getId());
            map.put("farmer",farmer.getFarmer());
            map.put("phone",farmer.getPhone());
            map.put("address",farmer.getAddress());
            map.put("farmCode",farmer.getFarmCode());
            if (ObjectUtil.isNotEmpty(eartagDeviceRegisters)){
                List farmerIds = new ArrayList();
                for (EartagDeviceRegister eartagDeviceRegister : eartagDeviceRegisters) {
                    farmerIds.add(eartagDeviceRegister.getDeviceCode());
                }
                map.put("data", eartagEartagRegister2Mapper.selectList(new QueryWrapper<EartagEartagRegister2>().in("last_device",farmerIds)));
            }
            endList.add(map);
        }

        return new Result(ResultCode.SUCCESS,endList);
    }


}
