package com.huimv.eartag2.api.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.eartag2.common.utils.DateUtil;
import com.huimv.eartag2.common.utils.Result;
import com.huimv.eartag2.common.utils.ResultCode;
import com.huimv.eartag2.api.mapper.BaseFarmMapper;
import com.huimv.eartag2.api.mapper.EartagDeviceRegisterMapper;
import com.huimv.eartag2.api.pojo.BaseFarm;
import com.huimv.eartag2.api.pojo.EartagDeviceRegister;
import com.huimv.eartag2.api.pojo.SysBaseConfig;
import com.huimv.eartag2.api.mapper.SysBaseConfigMapper;
import com.huimv.eartag2.api.service.SysBaseConfigService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.eartag2.api.vo.SysBaseConfigVo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zn
 * @since 2022-07-19
 */
@Service
public class SysBaseConfigServiceImpl extends ServiceImpl<SysBaseConfigMapper, SysBaseConfig> implements SysBaseConfigService {
    @Resource
    private SysBaseConfigMapper sysBaseConfigMapper;
    @Resource
    private BaseFarmMapper baseFarmMapper;
    @Resource
    RabbitTemplate rabbitTemplate;
    @Resource
    private EartagDeviceRegisterMapper eartagDeviceRegisterMapper;
    @Override
    public Result add(List<Map<String, String>> mapList) {
        //SysBaseConfigEntity
        //存入数据库
        String fitId = UUID.randomUUID().toString();
        for (Map<String, String> stringStringMap : mapList) {
            SysBaseConfig sysBaseConfigEntity = new SysBaseConfig();
            sysBaseConfigEntity.setConfigName(stringStringMap.get("configName"));
            sysBaseConfigEntity.setConfigKey(stringStringMap.get("configKey"));
            sysBaseConfigEntity.setConfigValue(stringStringMap.get("configValue"));
            sysBaseConfigEntity.setRowId(fitId);
            sysBaseConfigMapper.insert(sysBaseConfigEntity);
        }
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result remove(List<String> id) {
        //先根据row_id查询出对应的数据
        List<SysBaseConfig> sysBaseConfigs = sysBaseConfigMapper.selectByRowIdList(id);
        List<Integer> ids = sysBaseConfigs.stream().map(SysBaseConfig::getId).collect(Collectors.toList());
        sysBaseConfigMapper.deleteBatchIds(ids);
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result update(List<Map<String, String>> maps) {
        String rowId = maps.get(0).get("id");

        List<SysBaseConfig> sysBaseConfigs = sysBaseConfigMapper.selectSysBaseConfig(rowId);

        HashMap<String, String> stringHashMap = new HashMap<>();
        for (Map<String, String> map : maps) {
            if ("farmName".equals(map.get("configKey"))){
                stringHashMap.put("farmName",map.get("configValue"));
            }
            if ("ip".equals(map.get("configKey"))){
                stringHashMap.put("ip",map.get("configValue"));
            }
            if ("port".equals(map.get("configKey"))){
                stringHashMap.put("port",map.get("configValue"));
            }
            if ("farmCode".equals(map.get("configKey"))){
                stringHashMap.put("farmCode",map.get("configValue"));
            }
            if ("areaName".equals(map.get("configKey"))){
                stringHashMap.put("areaName",map.get("configValue"));
            }
        }

        for (SysBaseConfig sysBaseConfig : sysBaseConfigs) {
            if ("farmCode".equals(sysBaseConfig.getConfigKey())){
                sysBaseConfig.setConfigValue(stringHashMap.get("farmCode"));
            }
            if ("ip".equals(sysBaseConfig.getConfigKey())){
                sysBaseConfig.setConfigValue(stringHashMap.get("ip"));
            }
            if ("farmName".equals(sysBaseConfig.getConfigKey())){
                sysBaseConfig.setConfigValue(stringHashMap.get("farmName"));
            }
            if ("port".equals(sysBaseConfig.getConfigKey())){
                sysBaseConfig.setConfigValue(stringHashMap.get("port"));
            }
            if ("areaName".equals(sysBaseConfig.getConfigKey())){
                sysBaseConfig.setConfigValue(stringHashMap.get("areaName"));
            }
            sysBaseConfigMapper.updateById(sysBaseConfig);
        }
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result list(Map<String, String> paramsMap) {
        //先分页查询

       // Page<SysBaseConfig> page = new Page<>(pageNo, Integer.valueOf(paramsMap.get("pageSize")));
        QueryWrapper<SysBaseConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(StringUtils.isNotBlank(paramsMap.get("farmName")),"config_value", paramsMap.get("farmName"));
        queryWrapper.like(StringUtils.isNotBlank(paramsMap.get("farmCode")),"config_value", paramsMap.get("farmCode"));
        queryWrapper.like(StringUtils.isNotBlank(paramsMap.get("ip")),"config_value", paramsMap.get("ip"));
        //创建时间降序
        queryWrapper.orderByDesc("id");

        //获取到的只是当前传入参数的值的那条数据、需要再根据每个对象中的row_id获取到当前一组的数据、再进行拼装
        List<SysBaseConfig> pageBizDeviceRegister = sysBaseConfigMapper.selectList(queryWrapper);

        List<String> rowIds = pageBizDeviceRegister.stream().map(SysBaseConfig::getRowId).collect(Collectors.toList());

        //根据row_id分组查询数据
        List<SysBaseConfig> sysBaseConfigs = sysBaseConfigMapper.selectByRowIdList(rowIds);
        //将查询出来的数据  按照row_id进行分组
        Map<String, List<SysBaseConfig>> groupResult = sysBaseConfigs.stream().collect(Collectors.groupingBy(SysBaseConfig::getRowId));

        Set<String> strings = groupResult.keySet();

        //最终返回的拼装好的数据
        ArrayList<SysBaseConfigVo> resultList = new ArrayList<>();
        strings.forEach(rId->{
            List<SysBaseConfig> sysBaseConfigEntities = groupResult.get(rId);
            SysBaseConfigVo sysBaseConfigVo = convertSysBaseConfigVo(sysBaseConfigEntities);
            resultList.add(sysBaseConfigVo);
        });

        return new Result(ResultCode.SUCCESS,resultList);

    }

    @Override
    public Result getDateById(String ids) {
        List<SysBaseConfig> sysBaseConfigs = sysBaseConfigMapper.selectSysBaseConfig(ids);
        SysBaseConfigVo sysBaseConfigVo = convertSysBaseConfigVo(sysBaseConfigs);
        return new Result(ResultCode.SUCCESS,sysBaseConfigVo);
    }

    //同步牧场配置消息
    public final static String EXCHANGE_SYNC_SYS_CONFIG = "topic.syncSysConfig.exchange";
    public final static String ROUTING_KEY_SYNC_SYS_CONFIG = "topic.syncSysConfig.routeKey";

//    @Override
    public Result syncCong_yang(Map<String,String> paramsMap) {
        String farmCode = paramsMap.get("farmId");
        //查询牧场信息
        QueryWrapper<SysBaseConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(farmCode),"config_value", farmCode);
        List<SysBaseConfig> sysBaseConfigs = sysBaseConfigMapper.selectList(queryWrapper);
        String rowId = "";
        if (null == sysBaseConfigs || sysBaseConfigs.size() == 0){
            return new Result(ResultCode.FAIL);
        }else {
            rowId = sysBaseConfigs.get(0).getRowId();
        }
        List<SysBaseConfig> syncDate = sysBaseConfigMapper.selectSysBaseConfig(rowId);
        SysBaseConfigVo sysBaseConfigVo = convertSysBaseConfigVo(syncDate);
        paramsMap.put("farmName",sysBaseConfigVo.getFarmName());
        paramsMap.put("farmCode",sysBaseConfigVo.getFarmCode());
        paramsMap.put("ip",sysBaseConfigVo.getIp());
        paramsMap.put("port",sysBaseConfigVo.getPort());
        paramsMap.put("areaName",sysBaseConfigVo.getAreaName());
        rabbitTemplate.convertAndSend(EXCHANGE_SYNC_SYS_CONFIG,ROUTING_KEY_SYNC_SYS_CONFIG,paramsMap);
        System.out.println("--------------------------------------- 发送设备请求原始数据 >>"+paramsMap);
        //更新同步时间
        String chipId = paramsMap.get("chipId");
        QueryWrapper<EartagDeviceRegister> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("chip_id",chipId);
        List<EartagDeviceRegister> eartagDeviceRegisters = eartagDeviceRegisterMapper.selectList(queryWrapper1);
        EartagDeviceRegister eartagDeviceRegister = eartagDeviceRegisters.get(0).setSyncTime(new DateUtil().getTimestamp());
        eartagDeviceRegisterMapper.updateById(eartagDeviceRegister);
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result syncCong(Map<String,String> paramsMap) {
        String farmCode = paramsMap.get("farmId");
        System.out.println("paramsMap>>"+paramsMap.toString());
        String chipId = paramsMap.get("chipId");
        QueryWrapper<EartagDeviceRegister> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("chip_id",chipId);
        List<EartagDeviceRegister> eartagDeviceRegisters = eartagDeviceRegisterMapper.selectList(queryWrapper1);
        Integer deviceType = eartagDeviceRegisters.get(0).getDeviceType();

        //查询牧场信息
        QueryWrapper<BaseFarm> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotBlank(farmCode),"farm_code", farmCode);
        List<BaseFarm> baseFarmList = baseFarmMapper.selectList(queryWrapper);
        // 应急写法
        BaseFarm baseFarm = baseFarmList.get(0);

        paramsMap.put("farmName",baseFarm.getFarmName());
        paramsMap.put("farmCode",baseFarm.getFarmCode());
        paramsMap.put("ip",baseFarm.getIp());
        paramsMap.put("port",baseFarm.getPort());
        paramsMap.put("deviceType",String.valueOf(deviceType));
        paramsMap.put("areaCode",baseFarm.getAreaCode());
        rabbitTemplate.convertAndSend(EXCHANGE_SYNC_SYS_CONFIG,ROUTING_KEY_SYNC_SYS_CONFIG,paramsMap);
        System.out.println("--------------------------------------- 发送设备请求原始数据 >>"+paramsMap);
        //更新同步时间
//        String chipId = paramsMap.get("chipId");
//        QueryWrapper<EartagDeviceRegister> queryWrapper1 = new QueryWrapper<>();
//        queryWrapper1.eq("chip_id",chipId);
//        List<EartagDeviceRegister> eartagDeviceRegisters = eartagDeviceRegisterMapper.selectList(queryWrapper1);
        EartagDeviceRegister eartagDeviceRegister = eartagDeviceRegisters.get(0).setSyncTime(new DateUtil().getTimestamp());
        eartagDeviceRegisterMapper.updateById(eartagDeviceRegister);
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result listWarning(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        QueryWrapper<SysBaseConfig> wrapper = new QueryWrapper<>();
        SysBaseConfig baseConfig = sysBaseConfigMapper.selectOne(wrapper.eq("config_value", farmId));
        Map endMap = new HashMap();

        if (ObjectUtil.isNotEmpty(baseConfig)){
            wrapper.clear();
            wrapper.eq("row_id",baseConfig.getRowId()).eq("config_key","eartagOfflineTime");
            SysBaseConfig eartagOfflineTime = sysBaseConfigMapper.selectOne(wrapper);
            endMap.put("eartagOfflineTime",eartagOfflineTime.getConfigValue());

            wrapper.clear();
            wrapper.eq("row_id",baseConfig.getRowId()).eq("config_key","deviceOfflineTime");
            SysBaseConfig deviceOfflineTime = sysBaseConfigMapper.selectOne(wrapper);
            endMap.put("deviceOfflineTime",deviceOfflineTime.getConfigValue());

            wrapper.clear();
            wrapper.eq("row_id",baseConfig.getRowId()).eq("config_key","eartagMaxTem");
            SysBaseConfig eartagMaxTem = sysBaseConfigMapper.selectOne(wrapper);
            endMap.put("eartagMaxTem",eartagMaxTem.getConfigValue());

            wrapper.clear();
            wrapper.eq("row_id",baseConfig.getRowId()).eq("config_key","eartagMinTem");
            SysBaseConfig eartagMinTem = sysBaseConfigMapper.selectOne(wrapper);
            endMap.put("eartagMinTem",eartagMinTem.getConfigValue());
        }else {
            endMap.put("eartagOfflineTime",0);
            endMap.put("deviceOfflineTime",0);
            endMap.put("eartagMaxTem",0);
            endMap.put("eartagMinTem",0);
        }




        return new Result(ResultCode.SUCCESS,endMap);
    }

    @Override
    public Result updateWarning(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String eartagMaxTem = paramsMap.get("eartagMaxTem");
        String eartagMinTem = paramsMap.get("eartagMinTem");
        String eartagOfflineTime = paramsMap.get("eartagOfflineTime");
        String deviceOfflineTime = paramsMap.get("deviceOfflineTime");

        QueryWrapper<SysBaseConfig> wrapper = new QueryWrapper<>();
        SysBaseConfig baseConfig = sysBaseConfigMapper.selectOne(wrapper.eq("config_value", farmId));
        String  rowId ="";
        if (ObjectUtil.isEmpty(baseConfig)){
             rowId = UUID.randomUUID().toString();
            SysBaseConfig sysBaseConfigEntity = new SysBaseConfig();
            sysBaseConfigEntity.setConfigName("牧场编码");
            sysBaseConfigEntity.setConfigKey("farmCode");
            sysBaseConfigEntity.setConfigValue(farmId);
            sysBaseConfigEntity.setRowId(rowId);
            sysBaseConfigMapper.insert(sysBaseConfigEntity);
        }else {
            rowId  = baseConfig.getRowId();
        }

        wrapper.clear();
        wrapper.eq("config_key","eartagMaxTem");
        wrapper.eq("row_id",rowId);
        SysBaseConfig eartagMaxTemSys = sysBaseConfigMapper.selectOne(wrapper);

        if (ObjectUtil.isNotEmpty(eartagMaxTemSys)){
            eartagMaxTemSys.setConfigValue(eartagMaxTem);
            sysBaseConfigMapper.updateById(eartagMaxTemSys);
        }else {
            SysBaseConfig newBaseConfig = new SysBaseConfig();
            newBaseConfig.setConfigKey("eartagMaxTem");
            newBaseConfig.setConfigName("耳标温度超过温度报警");
            newBaseConfig.setConfigValue(eartagMaxTem);
            newBaseConfig.setRowId(rowId);
            sysBaseConfigMapper.insert(newBaseConfig);
        }


        wrapper.clear();
        wrapper.eq("config_key","eartagMinTem");
        wrapper.eq("row_id",rowId);
        SysBaseConfig eartagMinTemSys = sysBaseConfigMapper.selectOne(wrapper);

        if (ObjectUtil.isNotEmpty(eartagMinTemSys)){
            eartagMinTemSys.setConfigValue(eartagMinTem);
            sysBaseConfigMapper.updateById(eartagMinTemSys);
        }else {
            SysBaseConfig newBaseConfig = new SysBaseConfig();
            newBaseConfig.setConfigKey("eartagMinTem");
            newBaseConfig.setConfigName("耳标温度低于温度报警");
            newBaseConfig.setConfigValue(eartagMinTem);
            newBaseConfig.setRowId(rowId);
            sysBaseConfigMapper.insert(newBaseConfig);
        }

        wrapper.clear();
        wrapper.eq("config_key","eartagOfflineTime");
        wrapper.eq("row_id",rowId);
        SysBaseConfig eartagOfflineTimeSys = sysBaseConfigMapper.selectOne(wrapper);

        if (ObjectUtil.isNotEmpty(eartagOfflineTimeSys)){
            eartagOfflineTimeSys.setConfigValue(eartagOfflineTime);
            sysBaseConfigMapper.updateById(eartagOfflineTimeSys);
        }else {
            SysBaseConfig newBaseConfig = new SysBaseConfig();
            newBaseConfig.setConfigKey("eartagOfflineTime");
            newBaseConfig.setConfigName("耳标离线时间（分钟）");
            newBaseConfig.setConfigValue(eartagOfflineTime);
            newBaseConfig.setRowId(rowId);
            sysBaseConfigMapper.insert(newBaseConfig);
        }


        wrapper.clear();
        wrapper.eq("config_key","deviceOfflineTime");
        wrapper.eq("row_id",rowId);
        SysBaseConfig deviceOfflineTimeSys = sysBaseConfigMapper.selectOne(wrapper);

        if (ObjectUtil.isNotEmpty(deviceOfflineTimeSys)){
            deviceOfflineTimeSys.setConfigValue(deviceOfflineTime);
            sysBaseConfigMapper.updateById(deviceOfflineTimeSys);
        }else {
            SysBaseConfig newBaseConfig = new SysBaseConfig();
            newBaseConfig.setConfigKey("deviceOfflineTime");
            newBaseConfig.setConfigName("采集器离线时间（分钟）");
            newBaseConfig.setConfigValue(deviceOfflineTime);
            newBaseConfig.setRowId(rowId);
            sysBaseConfigMapper.insert(newBaseConfig);
        }

        return new Result(10000,"修改成功",true);

    }



    private SysBaseConfigVo convertSysBaseConfigVo(List<SysBaseConfig> list){
        Map<String, String> map = list.stream().collect(Collectors.toMap(SysBaseConfig::getConfigKey, SysBaseConfig::getConfigValue,(item1,item2) ->item1));
        SysBaseConfigVo sysBaseConfigVo = new SysBaseConfigVo();
        sysBaseConfigVo.setId(list.get(0).getRowId());
        sysBaseConfigVo.setFarmCode(map.get("farmCode"));
        sysBaseConfigVo.setFarmName(map.get("farmName"));
        sysBaseConfigVo.setIp(map.get("ip"));
        sysBaseConfigVo.setPort(map.get("port"));
        sysBaseConfigVo.setAreaName(map.get("areaName"));
        return sysBaseConfigVo;

    }

}
